#ifndef _DYNAMIXEL2_H
#define _DYNAMIXEL2_H

#ifdef __cplusplus
extern "C" {
#endif

#include "dynamixel_common.h"

#define     DYN2_HEADER_OFF      0
#define     DYN2_RESERVED        3
#define     DYN2_ID_OFF          4
#define     DYN2_LENGTH_OFF      5
#define     DYN2_INST_OFF        7
#define     DYN2_ERROR_OFF       8
#define     DYN2_DATA_OFF        8

/**
 * \brief
 * 
 */
void dyn2_copy_packet(unsigned char *source, unsigned char *destination);
/**
 * \brief
 * 
 */
static inline unsigned char dyn2_get_id(unsigned char *packet)
{
  return packet[DYN2_ID_OFF];
}
/**
 * \brief
 * 
 */
static inline unsigned short int dyn2_get_length(unsigned char *packet)
{
  return (packet[DYN2_LENGTH_OFF]+packet[DYN2_LENGTH_OFF+1]*256);
}
/**
 * \brief
 * 
 */
static inline TDynInstruction dyn2_get_instruction(unsigned char *packet)
{
  return (TDynInstruction)packet[DYN2_INST_OFF];
}
/**
 * \brief
 * 
 */
unsigned char dyn2_check_checksum(unsigned char *packet);

// instruction packet
/*
typedef struct{
  unsigned char header[2];
  unsigned char id;
  unsigned char length;
  unsigned char instruction;
  unsigned char data[MAX_DATA_LENGTH];
  unsigned char checksum;
}TDynInst;
*/
/**
 * \brief
 * 
 */
unsigned char dyn2_convert_v1_inst_packet(unsigned char *source, unsigned char *destination);
/* ping instruction */
/**
 * \brief
 * 
 */
void dyn2_init_ping_packet(unsigned char *packet,unsigned char id);
/* read instruction */
/**
 * \brief
 * 
 */
void dyn2_init_read_packet(unsigned char *packet,unsigned char id,unsigned short int address,unsigned short int length);
/**
 * \brief
 * 
 */
static inline unsigned short int dyn2_get_read_length(unsigned char *packet)
{
  return (packet[DYN2_DATA_OFF+2]+packet[DYN2_DATA_OFF+3]*256);
}
/**
 * \brief
 * 
 */
static inline unsigned short int dyn2_get_read_address(unsigned char *packet)
{
  return (packet[DYN2_DATA_OFF]+packet[DYN2_DATA_OFF+1]*256);
}
/* write instruction */
/**
 * \brief
 * 
 */
void dyn2_init_write_packet(unsigned char *packet,unsigned char id,unsigned short int address,unsigned short int length,unsigned char *data);
/**
 * \brief
 * 
 */
static inline unsigned short int dyn2_get_write_address(unsigned char *packet)
{
  return (packet[DYN2_DATA_OFF]+packet[DYN2_DATA_OFF+1]*256);
}
/**
 * \brief
 * 
 */
static inline unsigned short int dyn2_get_write_length(unsigned char *packet)
{
  return (packet[DYN2_LENGTH_OFF]+packet[DYN2_LENGTH_OFF+1]*256)-5;
}
/**
 * \brief
 * 
 */
unsigned short int dyn2_get_write_data(unsigned char *packet,unsigned char *data);
/* registered write instruction */
/**
 * \brief
 * 
 */
void dyn2_init_reg_write_packet(unsigned char *packet,unsigned char id,unsigned short int address,unsigned short int length,unsigned char *data);
/**
 * \brief
 * 
 */
static inline unsigned short int dyn2_get_reg_write_address(unsigned char *packet)
{
  return (packet[DYN2_DATA_OFF]+packet[DYN2_DATA_OFF+1]*256);
}
/**
 * \brief
 * 
 */
static inline unsigned short int dyn2_get_reg_write_length(unsigned char *packet)
{
  return (packet[DYN2_LENGTH_OFF]+packet[DYN2_LENGTH_OFF+1]*256)-5;
}
/**
 * \brief
 * 
 */
unsigned short int dyn2_get_reg_write_data(unsigned char *packet,unsigned char *data);
/* action instruction */
/**
 * \brief
 * 
 */
void dyn2_init_action_packet(unsigned char *packet);
/* reset instruction */
/**
 * \brief
 * 
 */
void dyn2_init_reset_packet(unsigned char *packet,unsigned char id,unsigned char mode);
/* sync write instruction */
/**
 * \brief
 * 
 */
void dyn2_init_sync_write_packet(unsigned char *packet,unsigned char num_servos,unsigned char *servo_ids,unsigned short int address,unsigned short int length,unsigned char * const data[]);
/**
 * \brief
 * 
 */
unsigned char dyn2_sync_write_id_present(unsigned char *packet,unsigned char id,unsigned short int *address,unsigned short int *length,unsigned char *data);
/* sync read instruction */
/**
 * \brief
 * 
 */
void dyn2_init_sync_read_packet(unsigned char *packet,unsigned char num_servos,unsigned char *servo_ids,unsigned short int address,unsigned short int length);
/**
 * \brief
 * 
 */
unsigned char dyn2_sync_read_id_present(unsigned char *packet,unsigned char id,unsigned short int *address,unsigned short int *length);
/* bulk read instruction */
/**
 * \brief
 * 
 */
void dyn2_init_bulk_read_packet(unsigned char *packet,unsigned char num_servos,unsigned char *servo_ids,unsigned short int *address,unsigned short int *length);
/**
 * \brief
 * 
 */
unsigned char dyn2_bulk_read_id_present(unsigned char *packet,unsigned char id,unsigned short int *address,unsigned short int *length);
/* bulk write instruction */
/**
 * \brief
 * 
 */
void dyn2_init_bulk_write_packet(unsigned char *packet,unsigned char num_servos,unsigned char *servo_ids,unsigned short int *address,unsigned short int *length,unsigned char * const data[]);
/**
 * \brief
 * 
 */
unsigned char dyn2_bulk_write_id_present(unsigned char *packet,unsigned char id,unsigned short int *address,unsigned short int *length,unsigned char *data);

// status packet
/*
typedef struct{
  unsigned char header[2];
  unsigned char id;
  unsigned char length;
  unsigned char error;
  unsigned char data[MAX_DATA_LENGTH];
  unsigned char checksum;
}TDynStatus;
*/

/**
 * \brief
 * 
 */
unsigned char dyn2_convert_v1_status_packet(TDynInstruction inst,unsigned char *source, unsigned char *destination);
/**
 * \brief
 * 
 */
void dyn2_init_status_packet(unsigned char *packet,unsigned char id,TDynError error,unsigned short int length,unsigned char *data);
/**
 * \brief
 * 
 */
static inline TDynError dyn2_get_status_error(unsigned char *packet)
{
  return (TDynError)packet[DYN2_DATA_OFF];
}
/* read instruction status packet */
/**
 * \brief
 * 
 */
unsigned short int dyn2_get_read_status_data(unsigned char *packet,unsigned char *data);
/**
 * \brief
 * 
 */
static inline unsigned char dyn2_get_read_status_id(unsigned char *packet)
{
  return packet[DYN2_ID_OFF];
}

#ifdef __cplusplus
}
#endif

#endif
