#ifndef _DYNAMIXEL_SLAVE_DEVICE_H
#define _DYNAMIXEL_SLAVE_DEVICE_H

#ifdef __cplusplus
extern "C" {
#endif

#include "dynamixel_slave_registers.h"
#include "dynamixel.h"
#include "dynamixel2.h"
#include "ram.h"

#ifndef MAX_DYN_SLAVE_REG_BUFFER_LEN
  #error "Please, specify the maximum registered buffer length with the MAX_DYN_SLAVE_REG_BUFFER_LEN macro"
#endif

/**
 * \brief
 *
 */
typedef struct 
{
  /**
   * \brief
   *
   */
  struct TDynamixelSlave *slave;
  /**
   * \brief
   *
   */
  unsigned char address;
  /**
   * \brief
   *
   */
  unsigned short int eeprom_base_address1;
  /**
   * \brief
   *
   */
  unsigned short int eeprom_base_address2;
  /**
   * \brief
   *
   */
  unsigned char return_delay;
  /**
   * \brief
   *
   */
  return_level_t return_level;
  /**
   * \brief
   *
   */
  unsigned short int reg_address;
  /**
   * \brief
   *
   */
  unsigned short int reg_length;
  /**
   * \brief
   *
   */
  unsigned char reg_buffer[MAX_DYN_SLAVE_REG_BUFFER_LEN];
  /**
   * \brief
   *
   */
  unsigned short int sync_bulk_address;
  /**
   * \brief
   *
   */
  unsigned short int sync_bulk_length;
  /**
   * \brief
   *
   */
  unsigned char sync_bulk_prev_id;
  /**
   * \brief
   *
   */
  unsigned char sync_read_pending;
  /**
   * \brief
   *
   */
  unsigned char bulk_read_pending;
  /**
   * \brief
   *
   */
  TMemory memory;
  /**
   * \brief
   *
   */
  TMemModule mem_module;
  /**
   * \brief
   *
   */
  void (*on_ping)(void);
  /**
   * \brief
   *
   */
  unsigned char (*on_read)(unsigned short int address,unsigned short int length,unsigned char *data);
  /**
   * \brief
   *
   */
  unsigned char (*on_write)(unsigned short int address,unsigned short int length,unsigned char *data);
  /**
   * \brief
   *
   */
  unsigned char (*on_reset)(void);
}TDynamixelSlaveDevice;

/* public functions */
/**
 * \brief
 *
 */
unsigned char dyn_slave_device_init(TDynamixelSlaveDevice *device,unsigned char address,unsigned char eeprom_base_address1,unsigned char eeprom_base_address2);
/**
 * \brief
 *
 */
static inline void dyn_slave_device_set_address(TDynamixelSlaveDevice *device,unsigned char address)
{
  if(device!=0x00000000)
    device->address=address;
}
/**
 * \brief
 *
 */
static inline unsigned char dyn_slave_device_get_address(TDynamixelSlaveDevice *device)
{
  if(device!=0x00000000)
    return device->address;
  else
    return 0xFF;
}
/**
 * \brief
 *
 */
static inline void dyn_slave_device_set_return_delay(TDynamixelSlaveDevice *device,unsigned char delay)
{
  if(device!=0x00000000)
    device->return_delay=delay;
}
/**
 * \brief
 *
 */
static inline unsigned char dyn_slave_device_get_return_delay(TDynamixelSlaveDevice *device)
{
  if(device!=0x00000000)
    return device->return_delay;
  else
    return 0xFF;
}
/**
 * \brief
 *
 */
static inline void dyn_slave_device_set_return_level(TDynamixelSlaveDevice *device,return_level_t level)
{
  if(device!=0x00000000)
    device->return_level=level;
}
/**
 * \brief
 *
 */
static inline return_level_t dyn_slave_device_get_return_level(TDynamixelSlaveDevice *device)
{
  if(device!=0x00000000)
    return device->return_level;
  else
    return return_all;
}
/**
 * \brief
 *
 */
unsigned char dyn_v1_slave_loop(TDynamixelSlaveDevice *device,unsigned char *rx_buffer,unsigned char *error,unsigned char *length, unsigned char *data);
/**
 * \brief
 *
 */
unsigned char dyn_v2_slave_loop(TDynamixelSlaveDevice *device,unsigned char *rx_buffer,unsigned char *error,unsigned short int *length, unsigned char *data);
/**
 * \brief
 *
 */
static inline TMemory *dyn_slave_device_get_memory(TDynamixelSlaveDevice *device)
{
  return &device->memory;
}

#ifdef __cplusplus
}
#endif

#endif
