#ifndef _HEAD_TRACKING_H
#define _HEAD_TRACKINH_H

#include "head_tracking_registers.h"
#include "motion_module.h"
#include "memory.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct{
  unsigned short int kp;
  unsigned short int ki;
  unsigned short int kd;
  long int prev_error;// 48|16
  long int integral_part;
  long int integral_clamp;
  long int max_angle;
  long int min_angle;
  long int target_angle;
}TJointControl;
 
typedef struct{
  TMotionModule mmodule;
  TMemory *memory;
  TMemModule mem_module;
  unsigned short int ram_base_address;
  unsigned short int eeprom_base_address;
  TJointControl tracking_pan;
  unsigned char pan_servo_id;
  TJointControl tracking_tilt;
  unsigned char tilt_servo_id;
  long int tracking_period;
  short int tracking_period_ms;
  unsigned char tracking_enabled;
}THeadMModule;

// public functions
unsigned char head_tracking_init(THeadMModule *joint,TMemory *memory,unsigned short int ram_base_address,unsigned short int eeprom_base_address);
TMotionModule *head_tracking_get_module(THeadMModule *head);
void head_tracking_start(THeadMModule *joint);
void head_tracking_stop(THeadMModule *joint);
unsigned char head_is_tracking(THeadMModule *joint);
void head_tracking_set_pan_range(THeadMModule *joint,short int max_angle,short int min_angle);
void head_tracking_get_pan_range(THeadMModule *joint,short int *max_angle,short int *min_angle);
void head_tracking_set_pan_pid(THeadMModule *joint,unsigned short int kp,unsigned short int ki,unsigned short int kd, unsigned short int i_clamp);
void head_tracking_get_pan_pid(THeadMModule *joint,unsigned short int *kp,unsigned short int *ki,unsigned short int *kd, unsigned short int *i_clamp);
void head_tracking_set_tilt_range(THeadMModule *joint,short int max_angle,short int min_angle);
void head_tracking_get_tilt_range(THeadMModule *joint,short int *max_angle,short int *min_angle);
void head_tracking_set_tilt_pid(THeadMModule *joint,unsigned short int kp,unsigned short int ki,unsigned short int kd, unsigned short int i_clamp);
void head_tracking_get_tilt_pid(THeadMModule *joint,unsigned short int *kp,unsigned short int *ki,unsigned short int *kd, unsigned short int *i_clamp);
void head_tracking_set_pan_pan(THeadMModule *joint,short int target);
void head_tracking_set_tilt_target(THeadMModule *joint,short int target);
void head_tracking_set_pan_servo_id(THeadMModule *joint,unsigned char servo_id);
void head_tracking_set_tilt_servo_id(THeadMModule *joint,unsigned char servo_id);

#ifdef __cplusplus
}
#endif

#endif
