#ifndef _HEAD_TRACKING_REGISTERS_H
#define _HEAD_TRACKING_REGISTERS_H

#define RAM_HEAD_TRACKING_LENGTH                    13

#define HEAD_TRACKING_CONTROL_OFFSET                0 // bit 7 | bit 6 | bit 5 |  bit 4   | bit 3 | bit 2 |     bit 1     |      bit 0
                                                      //       |       |       | tracking |       |       | stop tracking | start tracking
  #define HEAD_TRACKING_START                       0x01
  #define HEAD_TRACKING_STOP                        0x02
  #define HEAD_TRACKING_RUNNING                     0x10
#define HEAD_TRACKING_MAX_PAN_OFFSET                1
#define HEAD_TRACKING_MIN_PAN_OFFSET                3
#define HEAD_TRACKING_PAN_TARGET_OFFSET             5
#define HEAD_TRACKING_MAX_TILT_OFFSET               7
#define HEAD_TRACKING_MIN_TILT_OFFSET               9
#define HEAD_TRACKING_TILT_TARGET_OFFSET            11

#define EEPROM_HEAD_TRACKING_LENGTH                 18

#define HEAD_TRACKING_PAN_P_GAIN_OFFSET             0
#define HEAD_TRACKING_PAN_I_GAIN_OFFSET             2
#define HEAD_TRACKING_PAN_D_GAIN_OFFSET             4
#define HEAD_TRACKING_PAN_I_CLAMP_OFFSET            6
#define HEAD_TRACKING_TILT_P_GAIN_OFFSET            8
#define HEAD_TRACKING_TILT_I_GAIN_OFFSET            10
#define HEAD_TRACKING_TILT_D_GAIN_OFFSET            12
#define HEAD_TRACKING_TILT_I_CLAMP_OFFSET           14
#define HEAD_TRACKING_PAN_SERVO_ID_OFFSET           16
#define HEAD_TRACKING_TILT_SERVO_ID_OFFSET          17

#define head_tracking_eeprom_data(name,section_name,base_address,DEFAULT_PAN_P_GAIN,DEFAULT_PAN_I_GAIN,DEFAULT_PAN_D_GAIN,DEFAULT_PAN_I_CLAMP,DEFAULT_TILT_P_GAIN,DEFAULT_TILT_I_GAIN,DEFAULT_TILT_D_GAIN,DEFAULT_TILT_I_CLAMP,DEFAULT_PAN_SERVO_ID,DEFAULT_TILT_SERVO_ID) \
unsigned short int name##_eeprom_data[] __attribute__ ((section (section_name))) __attribute__ ((aligned (4)))= {\
  DEFAULT_PAN_P_GAIN&0x00FF,base_address+HEAD_TRACKING_PAN_P_GAIN_OFFSET, \
  (DEFAULT_PAN_P_GAIN>>8)&0x00FF,base_address+HEAD_TRACKING_PAN_P_GAIN_OFFSET+1, \
  DEFAULT_PAN_I_GAIN&0x00FF,base_address+HEAD_TRACKING_PAN_I_GAIN_OFFSET, \
  (DEFAULT_PAN_I_GAIN>>8)&0x00FF,base_address+HEAD_TRACKING_PAN_I_GAIN_OFFSET+1, \
  DEFAULT_PAN_D_GAIN&0x00FF,base_address+HEAD_TRACKING_PAN_D_GAIN_OFFSET, \
  (DEFAULT_PAN_D_GAIN>>8)&0x00FF,base_address+HEAD_TRACKING_PAN_D_GAIN_OFFSET+1, \
  DEFAULT_PAN_I_CLAMP&0x00FF,base_address+HEAD_TRACKING_PAN_I_CLAMP_OFFSET, \
  (DEFAULT_PAN_I_CLAMP>>8)&0x00FF,base_address+HEAD_TRACKING_PAN_I_CLAMP_OFFSET+1, \
  DEFAULT_TILT_P_GAIN&0x00FF,base_address+HEAD_TRACKING_TILT_P_GAIN_OFFSET, \
  (DEFAULT_TILT_P_GAIN>>8)&0x00FF,base_address+HEAD_TRACKING_TILT_P_GAIN_OFFSET+1, \
  DEFAULT_TILT_I_GAIN&0x00FF,base_address+HEAD_TRACKING_TILT_I_GAIN_OFFSET, \
  (DEFAULT_TILT_I_GAIN>>8)&0x00FF,base_address+HEAD_TRACKING_TILT_I_GAIN_OFFSET+1, \
  DEFAULT_TILT_D_GAIN&0x00FF,base_address+HEAD_TRACKING_TILT_D_GAIN_OFFSET, \
  (DEFAULT_TILT_D_GAIN>>8)&0x00FF,base_address+HEAD_TRACKING_TILT_D_GAIN_OFFSET+1, \
  DEFAULT_TILT_I_CLAMP&0x00FF,base_address+HEAD_TRACKING_TILT_I_CLAMP_OFFSET, \
  (DEFAULT_TILT_I_CLAMP>>8)&0x00FF,base_address+HEAD_TRACKING_TILT_I_CLAMP_OFFSET+1, \
  DEFAULT_PAN_SERVO_ID&0x00FF,base_address+HEAD_TRACKING_PAN_SERVO_ID_OFFSET, \
  DEFAULT_TILT_SERVO_ID&0x00FF,base_address+HEAD_TRACKING_TILT_SERVO_ID_OFFSET};


#endif

