#ifndef _WALK_H
#define _WALK_H

#ifdef __cplusplus
extern "C" {
#endif

#include "walk_registers.h"
#include "motion_module.h"
#include "memory.h"

typedef struct{
  TMotionModule mmodule;
  TMemory *memory;
  TMemModule mem_module;
  unsigned short int ram_base_address;
  unsigned short int eeprom_base_address;
  // aim control
  unsigned char aim_on;
  // internal motion variables
  float x_swap_phase_shift;
  float x_swap_amplitude;
  float x_swap_amplitude_shift;
  float x_move_phase_shift;
  float x_move_amplitude;
  float x_move_amplitude_shift;
  float y_swap_phase_shift;
  float y_swap_amplitude;
  float y_swap_amplitude_shift;
  float y_move_phase_shift;
  float y_move_amplitude;
  float y_move_amplitude_shift;
  float z_swap_phase_shift;
  float z_swap_amplitude;
  float z_swap_amplitude_shift;
  float z_move_phase_shift;
  float z_move_amplitude;
  float z_move_amplitude_shift;
  float a_move_phase_shift;
  float a_move_amplitude;
  float a_move_amplitude_shift;
  // internal offset variables
  float hip_pitch_offset;
  float pelvis_offset;
  float pelvis_swing;
  float arm_swing_gain;
  float x_offset;
  float y_offset;
  float z_offset;
  float r_offset;
  float p_offset;
  float a_offset;
  // internal time variables
  float time;
  float period_time;
  float x_swap_period_time;
  float x_move_period_time;
  float y_swap_period_time;
  float y_move_period_time;
  float z_swap_period_time;
  float z_move_period_time;
  float a_move_period_time;
  float ssp_time_start_left;
  float ssp_time_end_left;
  float ssp_time_start_right;
  float ssp_time_end_right;
  float phase_time1;
  float phase_time2;
  float phase_time3;
  // control variables
  unsigned char running;
  float walk_period;
  unsigned char (*leg_ik_function)(float *out, float x, float y, float z, float a, float b, float c);
  unsigned char right_hip_roll_servo_id;
  unsigned char right_hip_yaw_servo_id;
  unsigned char right_hip_pitch_servo_id;
  unsigned char right_knee_servo_id;
  unsigned char right_ankle_pitch_servo_id;
  unsigned char right_ankle_roll_servo_id;
  unsigned char right_shoulder_pitch_servo_id;
  unsigned char left_hip_roll_servo_id;
  unsigned char left_hip_yaw_servo_id;
  unsigned char left_hip_pitch_servo_id;
  unsigned char left_knee_servo_id;
  unsigned char left_ankle_pitch_servo_id;
  unsigned char left_ankle_roll_servo_id;
  unsigned char left_shoulder_pitch_servo_id;
}TWalkMModule;
 
// public functions
unsigned char walk_init(TWalkMModule *walk,TMemory *memory,unsigned short int ram_base_address,unsigned short int eeprom_base_address);
TMotionModule *walk_get_module(TWalkMModule *walk);
void walk_start(TWalkMModule *walk);
void walk_stop(TWalkMModule *walk);
unsigned char is_walk(TWalkMModule *walk);

#ifdef __cplusplus
}
#endif

#endif
