#include "head_tracking.h"
#include "head_tracking_registers.h"
#include "ram.h"

// provate functions
void head_tracking_read_cmd(THeadMModule *module,unsigned short int address,unsigned short int length,unsigned char *data)
{
  ram_read_table(module->memory,address,length,data);
}

void head_tracking_write_cmd(THeadMModule *module,unsigned short int address,unsigned short int length,unsigned char *data)
{
  unsigned short int kp,kd,ki,i_clamp,ram_offset,eeprom_offset;
  short int max,min,target;

  eeprom_offset=address-module->eeprom_base_address;
  if(ram_in_window(module->eeprom_base_address+HEAD_TRACKING_PAN_P_GAIN_OFFSET,8,address,length))
  {
    head_tracking_get_pan_pid(module,&kp,&kd,&ki,&i_clamp);
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_PAN_P_GAIN_OFFSET,address,length))
      kp=(kp&0xFF00)|data[HEAD_TRACKING_PAN_P_GAIN_OFFSET-eeprom_offset];
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_PAN_P_GAIN_OFFSET+1,address,length))
      kp=(kp&0x00FF)|(data[HEAD_TRACKING_PAN_P_GAIN_OFFSET+1-eeprom_offset]<<8);
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_PAN_I_GAIN_OFFSET,address,length))
      ki=(kp&0xFF00)|data[HEAD_TRACKING_PAN_I_GAIN_OFFSET-eeprom_offset];
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_PAN_I_GAIN_OFFSET+1,address,length))
      ki=(kp&0x00FF)|(data[HEAD_TRACKING_PAN_I_GAIN_OFFSET+1-eeprom_offset]<<8);
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_PAN_D_GAIN_OFFSET,address,length))
      kd=(kp&0xFF00)|data[HEAD_TRACKING_PAN_D_GAIN_OFFSET-eeprom_offset];
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_PAN_D_GAIN_OFFSET+1,address,length))
      kd=(kp&0x00FF)|(data[HEAD_TRACKING_PAN_D_GAIN_OFFSET+1-eeprom_offset]<<8);
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_PAN_I_CLAMP_OFFSET,address,length))
      i_clamp=(kp&0xFF00)|data[HEAD_TRACKING_PAN_I_CLAMP_OFFSET-eeprom_offset];
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_PAN_I_CLAMP_OFFSET+1,address,length))
      i_clamp=(kp&0x00FF)|(data[HEAD_TRACKING_PAN_I_CLAMP_OFFSET+1-eeprom_offset]<<8);
    head_tracking_set_pan_pid(module,kp,kd,ki,i_clamp);
  }
  if(ram_in_window(module->eeprom_base_address+HEAD_TRACKING_TILT_P_GAIN_OFFSET,8,address,length))
  {
    head_tracking_get_tilt_pid(module,&kp,&kd,&ki,&i_clamp);
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_TILT_P_GAIN_OFFSET,address,length))
      kp=(kp&0xFF00)|data[HEAD_TRACKING_TILT_P_GAIN_OFFSET-eeprom_offset];
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_TILT_P_GAIN_OFFSET+1,address,length))
      kp=(kp&0x00FF)|(data[HEAD_TRACKING_TILT_P_GAIN_OFFSET+1-eeprom_offset]<<8);
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_TILT_I_GAIN_OFFSET,address,length))
      ki=(kp&0xFF00)|data[HEAD_TRACKING_TILT_I_GAIN_OFFSET-eeprom_offset];
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_TILT_I_GAIN_OFFSET+1,address,length))
      ki=(kp&0x00FF)|(data[HEAD_TRACKING_TILT_I_GAIN_OFFSET+1-eeprom_offset]<<8);
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_TILT_D_GAIN_OFFSET,address,length))
      kd=(kp&0xFF00)|data[HEAD_TRACKING_TILT_D_GAIN_OFFSET-eeprom_offset];
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_TILT_D_GAIN_OFFSET+1,address,length))
      kd=(kp&0x00FF)|(data[HEAD_TRACKING_TILT_D_GAIN_OFFSET+1-eeprom_offset]<<8);
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_TILT_I_CLAMP_OFFSET,address,length))
      i_clamp=(kp&0xFF00)|data[HEAD_TRACKING_TILT_I_CLAMP_OFFSET-eeprom_offset];
    if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_TILT_I_CLAMP_OFFSET+1,address,length))
      i_clamp=(kp&0x00FF)|(data[HEAD_TRACKING_TILT_I_CLAMP_OFFSET+1-eeprom_offset]<<8);
    head_tracking_set_tilt_pid(module,kp,kd,ki,i_clamp);
  }
  if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_PAN_SERVO_ID_OFFSET,address,length))
    head_tracking_set_pan_servo_id(module,data[HEAD_TRACKING_PAN_SERVO_ID_OFFSET-eeprom_offset]);
  if(ram_in_range(module->eeprom_base_address+HEAD_TRACKING_TILT_SERVO_ID_OFFSET,address,length))
    head_tracking_set_tilt_servo_id(module,data[HEAD_TRACKING_TILT_SERVO_ID_OFFSET-eeprom_offset]);
 
  // head tracking control
  ram_offset=address-module->ram_base_address;
  if(ram_in_range(module->ram_base_address+HEAD_TRACKING_CONTROL_OFFSET,address,length))
  {
    if(data[HEAD_TRACKING_CONTROL_OFFSET-ram_offset]&HEAD_TRACKING_START)
      head_tracking_start(module);
    if(data[HEAD_TRACKING_CONTROL_OFFSET-ram_offset]&HEAD_TRACKING_STOP)
      head_tracking_stop(module);
  }
  if(ram_in_window(module->ram_base_address+HEAD_TRACKING_MAX_PAN_OFFSET,4,address,length))
  {
    head_tracking_get_pan_range(module,&max,&min);
    if(ram_in_range(module->ram_base_address+HEAD_TRACKING_MAX_PAN_OFFSET,address,length)) 
      max=(max&0xFF00)|data[HEAD_TRACKING_MAX_PAN_OFFSET-ram_offset];
    if(ram_in_range(module->ram_base_address+HEAD_TRACKING_MAX_PAN_OFFSET+1,address,length))
      max=(max&0x00FF)|(data[HEAD_TRACKING_MAX_PAN_OFFSET+1-ram_offset]<<8);
    if(ram_in_range(module->ram_base_address+HEAD_TRACKING_MIN_PAN_OFFSET,address,length)) 
      max=(max&0xFF00)|data[HEAD_TRACKING_MIN_PAN_OFFSET-ram_offset];
    if(ram_in_range(module->ram_base_address+HEAD_TRACKING_MIN_PAN_OFFSET+1,address,length))
      max=(max&0x00FF)|(data[HEAD_TRACKING_MIN_PAN_OFFSET+1-ram_offset]<<8);
    head_tracking_set_pan_range(module,max,min);
  }
  if(ram_in_window(module->ram_base_address+HEAD_TRACKING_MAX_TILT_OFFSET,4,address,length))
  {
    head_tracking_get_tilt_range(module,&max,&min);
    if(ram_in_range(module->ram_base_address+HEAD_TRACKING_MAX_TILT_OFFSET,address,length)) 
      max=(max&0xFF00)|data[HEAD_TRACKING_MAX_TILT_OFFSET-ram_offset];
    if(ram_in_range(module->ram_base_address+HEAD_TRACKING_MAX_TILT_OFFSET+1,address,length))
      max=(max&0x00FF)|(data[HEAD_TRACKING_MAX_TILT_OFFSET+1-ram_offset]<<8);
    if(ram_in_range(module->ram_base_address+HEAD_TRACKING_MIN_TILT_OFFSET,address,length)) 
      max=(max&0xFF00)|data[HEAD_TRACKING_MIN_TILT_OFFSET-ram_offset];
    if(ram_in_range(module->ram_base_address+HEAD_TRACKING_MIN_TILT_OFFSET+1,address,length))
      max=(max&0x00FF)|(data[HEAD_TRACKING_MIN_TILT_OFFSET+1-ram_offset]<<8);
    head_tracking_set_tilt_range(module,max,min);
  }
  if(ram_in_window(module->ram_base_address+HEAD_TRACKING_PAN_TARGET_OFFSET,2,address,length))
  {
    target=(module->tracking_pan.target_angle>>9);
    if(ram_in_range(module->ram_base_address+HEAD_TRACKING_PAN_TARGET_OFFSET,address,length))
    { 
      target=(target&0xFF00)|data[HEAD_TRACKING_PAN_TARGET_OFFSET-ram_offset];
      module->memory->data[module->ram_base_address+HEAD_TRACKING_PAN_TARGET_OFFSET]=data[HEAD_TRACKING_PAN_TARGET_OFFSET-ram_offset];
    }
    if(ram_in_range(module->ram_base_address+HEAD_TRACKING_PAN_TARGET_OFFSET+1,address,length))
    {
      target=(target&0x00FF)|(data[HEAD_TRACKING_PAN_TARGET_OFFSET+1-ram_offset]<<8);
      module->memory->data[module->ram_base_address+HEAD_TRACKING_PAN_TARGET_OFFSET]=data[HEAD_TRACKING_PAN_TARGET_OFFSET-ram_offset];
    }
    module->tracking_pan.target_angle=(target<<9);
  }
  if(ram_in_window(module->ram_base_address+HEAD_TRACKING_TILT_TARGET_OFFSET,2,address,length))
  {
    target=(module->tracking_tilt.target_angle>>9);
    if(ram_in_range(module->ram_base_address+HEAD_TRACKING_TILT_TARGET_OFFSET,address,length))
    { 
      target=(target&0xFF00)|data[HEAD_TRACKING_TILT_TARGET_OFFSET-ram_offset];
      module->memory->data[module->ram_base_address+HEAD_TRACKING_TILT_TARGET_OFFSET]=data[HEAD_TRACKING_TILT_TARGET_OFFSET-ram_offset];
    }
    if(ram_in_range(module->ram_base_address+HEAD_TRACKING_TILT_TARGET_OFFSET+1,address,length))
    {
      target=(target&0x00FF)|(data[HEAD_TRACKING_TILT_TARGET_OFFSET+1-ram_offset]<<8);
      module->memory->data[module->ram_base_address+HEAD_TRACKING_TILT_TARGET_OFFSET]=data[HEAD_TRACKING_TILT_TARGET_OFFSET-ram_offset];
    }
    module->tracking_tilt.target_angle=(target<<9);
  }
}

TMotionModule *head_tracking_get_module(THeadMModule *head)
{
  return &head->mmodule;
}

void head_tracking_set_period(void *module,unsigned short int period_ms)
{
  THeadMModule *head=(THeadMModule *)module;

  // load the current period 
  head->tracking_period=(period_ms<<16)/1000;
  head->tracking_period_ms=period_ms;
}

void head_tracking_set_module(void *module,unsigned char servo_id)
{

}

// motion manager process function
void head_tracking_pre_process(void *module)
{
  THeadMModule *head=(THeadMModule *)module;
  long int pan_error, tilt_error;
  long int derivative_pan=0, derivative_tilt=0;

  if(head->tracking_enabled)
  {
    head->memory->data[head->ram_base_address+HEAD_TRACKING_CONTROL_OFFSET]|=HEAD_TRACKING_RUNNING;
    if(mmanager_get_module(head->mmodule.manager,head->pan_servo_id)==MM_HEAD)
    {
      pan_error=head->tracking_pan.target_angle-head->mmodule.manager->servo_values[head->pan_servo_id].target_angle;
      head->tracking_pan.integral_part+=(pan_error*head->tracking_period)>>16;
      if(head->tracking_pan.integral_part>head->tracking_pan.integral_clamp)
        head->tracking_pan.integral_part=head->tracking_pan.integral_clamp;
      else if(head->tracking_pan.integral_part<-head->tracking_pan.integral_clamp)
        head->tracking_pan.integral_part=-head->tracking_pan.integral_clamp;
      derivative_pan=((pan_error-head->tracking_pan.prev_error)<<16)/head->tracking_period;
      head->mmodule.manager->servo_values[head->pan_servo_id].target_angle+=((head->tracking_pan.kp*pan_error)>>16);
      head->mmodule.manager->servo_values[head->pan_servo_id].target_angle+=((head->tracking_pan.ki*head->tracking_pan.integral_part)>>16);      
      head->mmodule.manager->servo_values[head->pan_servo_id].target_angle+=((head->tracking_pan.kd*derivative_pan)>>16);      
      if(head->mmodule.manager->servo_values[head->pan_servo_id].target_angle>head->tracking_pan.max_angle)
        head->mmodule.manager->servo_values[head->pan_servo_id].target_angle=head->tracking_pan.max_angle;
      else if(head->mmodule.manager->servo_values[head->pan_servo_id].target_angle<head->tracking_pan.min_angle)
        head->mmodule.manager->servo_values[head->pan_servo_id].target_angle=head->tracking_pan.min_angle;
      head->tracking_pan.prev_error=pan_error;
    }
    if(mmanager_get_module(head->mmodule.manager,head->tilt_servo_id)==MM_HEAD)
    { 
      tilt_error=head->tracking_tilt.target_angle-head->mmodule.manager->servo_values[head->tilt_servo_id].target_angle;
      head->tracking_tilt.integral_part+=(tilt_error*head->tracking_period)>>16;
      if(head->tracking_tilt.integral_part>head->tracking_tilt.integral_clamp)
        head->tracking_tilt.integral_part=head->tracking_tilt.integral_clamp;
      else if(head->tracking_tilt.integral_part<-head->tracking_tilt.integral_clamp)
        head->tracking_tilt.integral_part=-head->tracking_tilt.integral_clamp;
      derivative_tilt=((tilt_error-head->tracking_tilt.prev_error)<<16)/head->tracking_period;
      head->mmodule.manager->servo_values[head->tilt_servo_id].target_angle+=((head->tracking_tilt.kp*tilt_error)>>16);
      head->mmodule.manager->servo_values[head->tilt_servo_id].target_angle+=((head->tracking_tilt.ki*head->tracking_tilt.integral_part)>>16);
      head->mmodule.manager->servo_values[head->tilt_servo_id].target_angle+=((head->tracking_tilt.kd*derivative_tilt)>>16);
      if(head->mmodule.manager->servo_values[head->tilt_servo_id].target_angle>head->tracking_tilt.max_angle)
        head->mmodule.manager->servo_values[head->tilt_servo_id].target_angle=head->tracking_tilt.max_angle;
      else if(head->mmodule.manager->servo_values[head->tilt_servo_id].target_angle<head->tracking_tilt.min_angle)
        head->mmodule.manager->servo_values[head->tilt_servo_id].target_angle=head->tracking_tilt.min_angle;
      head->tracking_tilt.prev_error=tilt_error;
    }
  }
  else
    head->memory->data[head->ram_base_address+HEAD_TRACKING_CONTROL_OFFSET]&=(~HEAD_TRACKING_RUNNING);
}

// public functions
unsigned char head_tracking_init(THeadMModule *head,TMemory *memory,unsigned short int ram_base_address,unsigned short int eeprom_base_address)
{
  /* initialize private variables */
  head->tracking_pan.prev_error=0;
  head->tracking_pan.integral_part=0;
  head->tracking_pan.target_angle=0;
  head->tracking_tilt.prev_error=0;
  head->tracking_tilt.integral_part=0;
  head->tracking_tilt.target_angle=0;
  head->tracking_enabled=0x00;

  /* initialize the motion module */
  mmodule_init(&head->mmodule);
  head->mmodule.id=MM_HEAD;
  head->mmodule.set_period=head_tracking_set_period;
  head->mmodule.set_module=head_tracking_set_module;
  head->mmodule.pre_process=head_tracking_pre_process;
  head->mmodule.data=head;

  mem_module_init(&head->mem_module);
  head->mem_module.data=head;
  head->mem_module.write_cmd=(void(*)(void *,unsigned short int, unsigned short int,unsigned char *))head_tracking_write_cmd;
  head->mem_module.read_cmd=(void(*)(void *,unsigned short int, unsigned short int,unsigned char *))head_tracking_read_cmd;
  if(!mem_module_add_ram_segment(&head->mem_module,ram_base_address,RAM_HEAD_TRACKING_LENGTH))
    return 0x00;
  if(!mem_module_add_eeprom_segment(&head->mem_module,eeprom_base_address,EEPROM_HEAD_TRACKING_LENGTH))
    return 0x00;
  if(!mem_add_module(memory,&head->mem_module))
    return 0x00;
  head->ram_base_address=ram_base_address;
  head->eeprom_base_address=eeprom_base_address;
  head->memory=memory;

  return 0x01;
}

void head_tracking_start(THeadMModule *head)
{
  head->tracking_enabled=0x01;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_CONTROL_OFFSET]|=HEAD_TRACKING_RUNNING;
  head_tracking_set_pan_range(head,head->mmodule.manager->servo_configs[head->pan_servo_id]->ccw_angle_limit<<9,head->mmodule.manager->servo_configs[head->pan_servo_id]->cw_angle_limit<<9);
  head_tracking_set_pan_range(head,head->mmodule.manager->servo_configs[head->tilt_servo_id]->ccw_angle_limit<<9,head->mmodule.manager->servo_configs[head->tilt_servo_id]->cw_angle_limit<<9);
}

void head_tracking_stop(THeadMModule *head)
{
  head->tracking_enabled=0x00;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_CONTROL_OFFSET]&=(~HEAD_TRACKING_RUNNING);
}

unsigned char head_is_tracking(THeadMModule *head)
{
  return head->tracking_enabled;
}

void head_tracking_set_pan_range(THeadMModule *head,short int max_angle,short int min_angle)
{
  head->tracking_pan.max_angle=max_angle<<9;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_MAX_PAN_OFFSET]=max_angle&0x00FF;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_MAX_PAN_OFFSET+1]=(max_angle>>8)&0x00FF;
  head->tracking_pan.min_angle=min_angle<<9;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_MIN_PAN_OFFSET]=min_angle&0x00FF;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_MIN_PAN_OFFSET+1]=(min_angle>>8)&0x00FF;
}

void head_tracking_get_pan_range(THeadMModule *head,short int *max_angle,short int *min_angle)
{
  *max_angle=head->tracking_pan.max_angle>>9;
  *min_angle=head->tracking_pan.min_angle>>9;
}

void head_tracking_set_pan_pid(THeadMModule *head,unsigned short int kp,unsigned short int ki,unsigned short int kd, unsigned short int i_clamp)
{
  head->tracking_pan.kp=kp;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_PAN_P_GAIN_OFFSET]=kp&0x00FF;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_PAN_P_GAIN_OFFSET+1]=(kp>>8)&0x00FF;
  head->tracking_pan.ki=ki;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_PAN_I_GAIN_OFFSET]=ki&0x00FF;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_PAN_I_GAIN_OFFSET+1]=(ki>>8)&0x00FF;
  head->tracking_pan.kd=kd;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_PAN_D_GAIN_OFFSET]=kd&0x00FF;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_PAN_D_GAIN_OFFSET+1]=(kd>>8)&0x00FF;
  head->tracking_pan.integral_clamp=i_clamp<<9;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_PAN_I_CLAMP_OFFSET]=i_clamp&0x00FF;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_PAN_I_CLAMP_OFFSET+1]=(i_clamp>>8)&0x00FF;
}

void head_tracking_get_pan_pid(THeadMModule *head,unsigned short int *kp,unsigned short int *ki,unsigned short int *kd, unsigned short int *i_clamp)
{
  *kp=head->tracking_pan.kp;
  *ki=head->tracking_pan.ki;
  *kd=head->tracking_pan.kd;
  *i_clamp=head->tracking_pan.integral_clamp>>9;
}

void head_tracking_set_tilt_range(THeadMModule *head,short int max_angle,short int min_angle)
{
  head->tracking_tilt.max_angle=max_angle<<9;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_MAX_TILT_OFFSET]=max_angle&0x00FF;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_MAX_TILT_OFFSET+1]=(max_angle>>8)&0x00FF;
  head->tracking_tilt.min_angle=min_angle<<9;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_MAX_TILT_OFFSET]=min_angle&0x00FF;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_MAX_TILT_OFFSET+1]=(min_angle>>8)&0x00FF;
}

void head_tracking_get_tilt_range(THeadMModule *head,short int *max_angle,short int *min_angle)
{
  *max_angle=head->tracking_tilt.max_angle>>9;
  *min_angle=head->tracking_tilt.min_angle>>9;
}

void head_tracking_set_tilt_pid(THeadMModule *head,unsigned short int kp,unsigned short int ki,unsigned short int kd, unsigned short int i_clamp)
{
  head->tracking_tilt.kp=kp;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_TILT_P_GAIN_OFFSET]=kp&0x00FF;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_TILT_P_GAIN_OFFSET+1]=(kp>>8)&0x00FF;
  head->tracking_tilt.ki=ki;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_TILT_I_GAIN_OFFSET]=ki&0x00FF;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_TILT_I_GAIN_OFFSET+1]=(ki>>8)&0x00FF;
  head->tracking_tilt.kd=kd;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_TILT_D_GAIN_OFFSET]=kd&0x00FF;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_TILT_D_GAIN_OFFSET+1]=(kd>>8)&0x00FF;
  head->tracking_tilt.integral_clamp=i_clamp<<9;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_TILT_I_CLAMP_OFFSET]=i_clamp&0x00FF;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_TILT_I_CLAMP_OFFSET+1]=(i_clamp>>8)&0x00FF;
}

void head_tracking_get_tilt_pid(THeadMModule *head,unsigned short int *kp,unsigned short int *ki,unsigned short int *kd, unsigned short int *i_clamp)
{
  *kp=head->tracking_tilt.kp;
  *ki=head->tracking_tilt.ki;
  *kd=head->tracking_tilt.kd;
  *i_clamp=head->tracking_tilt.integral_clamp>>9;
}

void head_tracking_set_pan_target(THeadMModule *head,short int target)
{
  head->tracking_pan.target_angle=target<<9;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_PAN_TARGET_OFFSET]=target&0x00FF;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_PAN_TARGET_OFFSET+1]=(target>>8)&0x00FF;
}

void head_tracking_set_tilt_target(THeadMModule *head,short int target)
{
  head->tracking_tilt.target_angle=target<<9;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_TILT_TARGET_OFFSET]=target&0x00FF;
  head->memory->data[head->ram_base_address+HEAD_TRACKING_TILT_TARGET_OFFSET+1]=(target>>8)&0x00FF;
}

void head_tracking_set_pan_servo_id(THeadMModule *head,unsigned char servo_id)
{
  head->pan_servo_id=servo_id;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_PAN_SERVO_ID_OFFSET]=servo_id;
}

void head_tracking_set_tilt_servo_id(THeadMModule *head,unsigned char servo_id)
{
  head->tilt_servo_id=servo_id;
  head->memory->data[head->eeprom_base_address+HEAD_TRACKING_TILT_SERVO_ID_OFFSET]=servo_id;
}
