#include "walk.h"
#include "walk_registers.h"
#include "ram.h"
#include <math.h>

enum {PHASE0=0x00,PHASE1=0x40,PHASE2=0x80,PHASE3=0xC0};

// private functions

void walk_read_cmd(TWalkMModule *module,unsigned short int address,unsigned short int length,unsigned char *data)
{
  ram_read_table(module->memory,address,length,data);
}

void walk_write_cmd(TWalkMModule *module,unsigned short int address,unsigned short int length,unsigned char *data)
{
  unsigned short int i,ram_offset,eeprom_offset;

  // walk control
  ram_offset=address-module->ram_base_address;
  if(ram_in_range(module->ram_base_address+WALK_CONTROL,address,length))
  {
    if(data[WALK_CONTROL-ram_offset]&WALK_START)
      walk_start(module);
    if(data[WALK_CONTROL-ram_offset]&WALK_STOP)
      walk_stop(module);
  }
  if(ram_in_range(module->ram_base_address+WALK_STEP_FW_BW,address,length))
    module->memory->data[module->ram_base_address+WALK_STEP_FW_BW]=data[WALK_STEP_FW_BW-ram_offset];
  if(ram_in_range(module->ram_base_address+WALK_STEP_LEFT_RIGHT,address,length))
    module->memory->data[module->ram_base_address+WALK_STEP_LEFT_RIGHT]=data[WALK_STEP_LEFT_RIGHT-ram_offset];
  if(ram_in_range(module->ram_base_address+WALK_STEP_DIRECTION,address,length))
    module->memory->data[module->ram_base_address+WALK_STEP_DIRECTION]=data[WALK_STEP_DIRECTION-ram_offset];
  // walk parameters
  eeprom_offset=address-module->eeprom_base_address;
  if(ram_in_range(module->ram_base_address+WALK_R_HIP_ROLL_SERVO_ID,address,length))
    module->right_hip_roll_servo_id=data[WALK_R_HIP_ROLL_SERVO_ID-eeprom_offset];
  if(ram_in_range(module->ram_base_address+WALK_R_HIP_YAW_SERVO_ID,address,length))
    module->right_hip_yaw_servo_id=data[WALK_R_HIP_YAW_SERVO_ID-eeprom_offset];
  if(ram_in_range(module->ram_base_address+WALK_R_HIP_PITCH_SERVO_ID,address,length))
    module->right_hip_pitch_servo_id=data[WALK_R_HIP_PITCH_SERVO_ID-eeprom_offset];
  if(ram_in_range(module->ram_base_address+WALK_R_KNEE_SERVO_ID,address,length))
    module->right_knee_servo_id=data[WALK_R_KNEE_SERVO_ID-eeprom_offset];
  if(ram_in_range(module->ram_base_address+WALK_R_ANKLE_PITCH_SERVO_ID,address,length))
    module->right_ankle_pitch_servo_id=data[WALK_R_ANKLE_PITCH_SERVO_ID-eeprom_offset];
  if(ram_in_range(module->ram_base_address+WALK_R_ANKLE_ROLL_SERVO_ID,address,length))
    module->right_ankle_roll_servo_id=data[WALK_R_ANKLE_ROLL_SERVO_ID-eeprom_offset];
  if(ram_in_range(module->ram_base_address+WALK_R_SHOULDER_PITCH_SERVO_ID,address,length))
    module->right_shoulder_pitch_servo_id=data[WALK_R_SHOULDER_PITCH_SERVO_ID-eeprom_offset];
  if(ram_in_range(module->ram_base_address+WALK_L_HIP_ROLL_SERVO_ID,address,length))
    module->left_hip_roll_servo_id=data[WALK_L_HIP_ROLL_SERVO_ID-eeprom_offset];
  if(ram_in_range(module->ram_base_address+WALK_L_HIP_YAW_SERVO_ID,address,length))
    module->left_hip_yaw_servo_id=data[WALK_L_HIP_YAW_SERVO_ID-eeprom_offset];
  if(ram_in_range(module->ram_base_address+WALK_L_HIP_PITCH_SERVO_ID,address,length))
    module->left_hip_pitch_servo_id=data[WALK_L_HIP_PITCH_SERVO_ID-eeprom_offset];
  if(ram_in_range(module->ram_base_address+WALK_L_KNEE_SERVO_ID,address,length))
    module->left_knee_servo_id=data[WALK_L_KNEE_SERVO_ID-eeprom_offset];
  if(ram_in_range(module->ram_base_address+WALK_L_ANKLE_PITCH_SERVO_ID,address,length))
    module->left_ankle_pitch_servo_id=data[WALK_L_ANKLE_PITCH_SERVO_ID-eeprom_offset];
  if(ram_in_range(module->ram_base_address+WALK_L_ANKLE_ROLL_SERVO_ID,address,length))
    module->left_ankle_roll_servo_id=data[WALK_L_ANKLE_ROLL_SERVO_ID-eeprom_offset];
  if(ram_in_range(module->ram_base_address+WALK_L_SHOULDER_PITCH_SERVO_ID,address,length))
    module->left_shoulder_pitch_servo_id=data[WALK_L_SHOULDER_PITCH_SERVO_ID-eeprom_offset];
  for(i=0;i<EEPROM_WALK_LENGTH;i++)
    if(ram_in_range(module->eeprom_base_address+i,address,length))
      module->memory->data[module->eeprom_base_address+i]=data[i-eeprom_offset];
}

float wsin(float time, float period, float period_shift, float mag, float mag_shift)
{
  return mag*sin(((2.0*3.14159*time)/period)-period_shift)+mag_shift;
}

void update_param_time(TWalkMModule *walk)
{
  float ssp_ratio;
  float ssp_time;

  walk->period_time=((float)((short int)(walk->memory->data[walk->eeprom_base_address+WALK_PERIOD_TIME]+(walk->memory->data[walk->eeprom_base_address+WALK_PERIOD_TIME+1]<<8))));
  ssp_ratio=1.0-((float)walk->memory->data[walk->eeprom_base_address+WALK_DSP_RATIO])/256.0;

  ssp_time=walk->period_time*ssp_ratio;

  walk->x_swap_period_time=walk->period_time/2.0;
  walk->x_move_period_time=ssp_time;
  walk->y_swap_period_time=walk->period_time;
  walk->y_move_period_time=ssp_time;
  walk->z_swap_period_time=walk->period_time/2.0;
  walk->z_move_period_time=ssp_time/2.0;
  walk->a_move_period_time=ssp_time;

  walk->ssp_time_start_left=(1-ssp_ratio)*walk->period_time/4.0;
  walk->ssp_time_end_left=(1+ssp_ratio)*walk->period_time/4.0;
  walk->ssp_time_start_right=(3-ssp_ratio)*walk->period_time/4.0;
  walk->ssp_time_end_right=(3+ssp_ratio)*walk->period_time/4.0;

  walk->phase_time1=(walk->ssp_time_end_left+walk->ssp_time_start_left)/2.0;
  walk->phase_time2=(walk->ssp_time_start_right+walk->ssp_time_end_left)/2.0;
  walk->phase_time3=(walk->ssp_time_end_right+walk->ssp_time_start_right)/2.0;

  // m_pelvis_offset and m_pelvis_Swing in degrees
  walk->pelvis_offset=((float)walk->memory->data[walk->eeprom_base_address+WALK_PELVIS_OFFSET])*3.14159/1440.0;
  walk->pelvis_swing=walk->pelvis_offset*0.35;
  walk->arm_swing_gain=((float)walk->memory->data[walk->eeprom_base_address+WALK_ARM_SWING_GAIN])/32.0;
}

void update_param_move(TWalkMModule *walk)
{
  float target_x_amplitude;
  float target_y_amplitude;
  float target_a_amplitude;
  static float current_x_amplitude=0;
  static float current_y_amplitude=0;
  static float current_a_amplitude=0;

  target_x_amplitude=((float)((char)walk->memory->data[walk->ram_base_address+WALK_STEP_FW_BW]));
  // change longitudinal and transversal velocities to get to the target ones
  if(current_x_amplitude<target_x_amplitude)
  {
    current_x_amplitude+=(((float)walk->memory->data[walk->eeprom_base_address+WALK_MAX_VEL])*walk->period_time)/1000.0;
    if(current_x_amplitude>target_x_amplitude)
      current_x_amplitude=target_x_amplitude;
  }
  else if(current_x_amplitude>target_x_amplitude)
  {
    current_x_amplitude-=(((float)walk->memory->data[walk->eeprom_base_address+WALK_MAX_VEL])*walk->period_time)/1000.0;
    if(current_x_amplitude<target_x_amplitude)
      current_x_amplitude=target_x_amplitude;
  }
  walk->x_move_amplitude=current_x_amplitude;
  walk->x_swap_amplitude=current_x_amplitude*((float)walk->memory->data[walk->eeprom_base_address+WALK_STEP_FW_BW_RATIO])/256.0;

  // Right/Left
  target_y_amplitude=((float)((char)walk->memory->data[walk->ram_base_address+WALK_STEP_LEFT_RIGHT]));
  if(current_y_amplitude<target_y_amplitude)
  {
    current_y_amplitude+=(((float)walk->memory->data[walk->eeprom_base_address+WALK_MAX_VEL])*walk->period_time)/1000.0;
    if(current_y_amplitude>target_y_amplitude)
      current_y_amplitude=target_y_amplitude;
  }
  else if(current_y_amplitude>target_y_amplitude)
  {
    current_y_amplitude-=(((float)walk->memory->data[walk->eeprom_base_address+WALK_MAX_VEL])*walk->period_time)/1000.0;
    if(current_y_amplitude<target_y_amplitude)
      current_y_amplitude=target_y_amplitude;
  }
  walk->y_move_amplitude=current_y_amplitude/2.0;
  if(walk->y_move_amplitude>0)
    walk->y_move_amplitude_shift=walk->y_move_amplitude;
  else
    walk->y_move_amplitude_shift=-walk->y_move_amplitude;
  walk->y_swap_amplitude=((float)walk->memory->data[walk->eeprom_base_address+WALK_SWING_RIGHT_LEFT])+walk->y_move_amplitude_shift*0.04;

  walk->z_move_amplitude=((float)walk->memory->data[walk->eeprom_base_address+WALK_FOOT_HEIGHT])/2.0;
  walk->z_move_amplitude_shift=walk->z_move_amplitude / 2.0;
  walk->z_swap_amplitude=((float)walk->memory->data[walk->eeprom_base_address+WALK_SWING_TOP_DOWN]);
  walk->z_swap_amplitude_shift=walk->z_swap_amplitude;

  // Direction
  target_a_amplitude=((float)((char)walk->memory->data[walk->ram_base_address+WALK_STEP_DIRECTION]))/8.0;
  if(current_a_amplitude<target_a_amplitude)
  {
    current_a_amplitude+=(((float)walk->memory->data[walk->eeprom_base_address+WALK_MAX_ROT_VEL])*walk->period_time)/8000.0;
    if(current_a_amplitude>target_a_amplitude)
      current_a_amplitude=target_a_amplitude;
  }
  else if(current_a_amplitude>target_a_amplitude)
  {
    current_a_amplitude-=(((float)walk->memory->data[walk->eeprom_base_address+WALK_MAX_ROT_VEL])*walk->period_time)/8000.0;
    if(current_a_amplitude<target_a_amplitude)
      current_a_amplitude=target_a_amplitude;
  }

  if(walk->aim_on==0x00)
  {
    walk->a_move_amplitude=current_a_amplitude*3.14159/180.0/2.0;
    if(walk->a_move_amplitude>0)
      walk->a_move_amplitude_shift=walk->a_move_amplitude;
    else
      walk->a_move_amplitude_shift=-walk->a_move_amplitude;
  }
  else
  {
    walk->a_move_amplitude=-current_a_amplitude*3.14159/180.0/2.0;
    if(walk->a_move_amplitude>0)
      walk->a_move_amplitude_shift=-walk->a_move_amplitude;
    else
      walk->a_move_amplitude_shift=walk->a_move_amplitude;
  }
}

void update_param_balance(TWalkMModule *walk)
{
  walk->x_offset=((float)((char)walk->memory->data[walk->eeprom_base_address+WALK_X_OFFSET]));
  walk->y_offset=((float)((char)walk->memory->data[walk->eeprom_base_address+WALK_Y_OFFSET]));
  walk->z_offset=((float)((char)walk->memory->data[walk->eeprom_base_address+WALK_Z_OFFSET]));
  walk->r_offset=((float)((char)walk->memory->data[walk->eeprom_base_address+WALK_ROLL_OFFSET]))*3.14159/1440.0;// (r_offset/8)*(pi/180)
  walk->p_offset = ((float)((char)walk->memory->data[walk->eeprom_base_address+WALK_PITCH_OFFSET]))*3.14159/1440.0;// (p_offset/8)*(pi/180)
  walk->a_offset = ((float)((char)walk->memory->data[walk->eeprom_base_address+WALK_YAW_OFFSET]))*3.14159/1440.0;// (a_offset/8)*(pi/180)
  walk->hip_pitch_offset = ((float)((short int)(walk->memory->data[walk->eeprom_base_address+WALK_HIP_PITCH_OFFSET]+(walk->memory->data[walk->eeprom_base_address+WALK_HIP_PITCH_OFFSET+1]<<8))))/1024.0;
}

// motion manager interface functions
void walk_pre_process(void *module)
{
  TWalkMModule *walk=(TWalkMModule *)module;
  float x_swap,y_swap,z_swap,a_swap,b_swap,c_swap;
  float x_move_r,y_move_r,z_move_r,a_move_r,b_move_r,c_move_r;
  float x_move_l,y_move_l,z_move_l,a_move_l,b_move_l,c_move_l;
  float pelvis_offset_r,pelvis_offset_l;
  float angle[14],ep[12];

  if(walk->time==0)
  {
    update_param_time(walk);
    walk->memory->data[walk->ram_base_address+WALK_CONTROL]&=(~WALK_CURRENT_PHASE);
    walk->memory->data[walk->ram_base_address+WALK_CONTROL]|=PHASE0;
    if(walk->running==0x00)
    {
      if(walk->x_move_amplitude==0 && walk->y_move_amplitude==0 && walk->a_move_amplitude==0)
        walk->memory->data[walk->ram_base_address+WALK_CONTROL]&=(~WALK_RUNNING);
      else
      {
        walk->memory->data[walk->ram_base_address+WALK_STEP_FW_BW]=0.0;
        walk->memory->data[walk->ram_base_address+WALK_STEP_LEFT_RIGHT]=0.0;
        walk->memory->data[walk->ram_base_address+WALK_STEP_DIRECTION]=0.0;
      }
    }
  }
  else if(walk->time>=(walk->phase_time1-walk->walk_period/2.0) && walk->time<(walk->phase_time1+walk->walk_period/2.0))
  {
    update_param_move(walk);
    walk->memory->data[walk->ram_base_address+WALK_CONTROL]&=(~WALK_CURRENT_PHASE);
    walk->memory->data[walk->ram_base_address+WALK_CONTROL]|=PHASE1;
  }
  else if(walk->time>=(walk->phase_time2-walk->walk_period/2.0) && walk->time<(walk->phase_time2+walk->walk_period/2.0))
  {
    update_param_time(walk);
    walk->time=walk->phase_time2;
    walk->memory->data[walk->ram_base_address+WALK_CONTROL]&=(~WALK_CURRENT_PHASE);
    walk->memory->data[walk->ram_base_address+WALK_CONTROL]|=PHASE2;
    if(walk->running==0x00)
    {
      if(walk->x_move_amplitude==0 && walk->y_move_amplitude==0 && walk->a_move_amplitude==0)
        walk->memory->data[walk->ram_base_address+WALK_CONTROL]&=~WALK_RUNNING;
      else
      {
        walk->memory->data[walk->ram_base_address+WALK_STEP_FW_BW]=0.0;
        walk->memory->data[walk->ram_base_address+WALK_STEP_LEFT_RIGHT]=0.0;
        walk->memory->data[walk->ram_base_address+WALK_STEP_DIRECTION]=0.0;
      }
    }
  }
  else if(walk->time>=(walk->phase_time3-walk->walk_period/2.0) && walk->time<(walk->phase_time3+walk->walk_period/2.0))
  {
    update_param_move(walk);
    walk->memory->data[walk->ram_base_address+WALK_CONTROL]&=(~WALK_CURRENT_PHASE);
    walk->memory->data[walk->ram_base_address+WALK_CONTROL]|=PHASE3;
  }
  update_param_balance(walk);

  // Compute endpoints
  x_swap=wsin(walk->time,walk->x_swap_period_time,walk->x_swap_phase_shift,walk->x_swap_amplitude,walk->x_swap_amplitude_shift);
  y_swap=wsin(walk->time,walk->y_swap_period_time,walk->y_swap_phase_shift,walk->y_swap_amplitude,walk->y_swap_amplitude_shift);
  z_swap=wsin(walk->time,walk->z_swap_period_time,walk->z_swap_phase_shift,walk->z_swap_amplitude,walk->z_swap_amplitude_shift);
  a_swap=0;
  b_swap=0;
  c_swap=0;
  if(walk->time<=walk->ssp_time_start_left)
  {
    x_move_l=wsin(walk->ssp_time_start_left,walk->x_move_period_time,walk->x_move_phase_shift+2.0*3.14159/walk->x_move_period_time*walk->ssp_time_start_left,walk->x_move_amplitude,walk->x_move_amplitude_shift);
    y_move_l=wsin(walk->ssp_time_start_left,walk->y_move_period_time,walk->y_move_phase_shift+2.0*3.14159/walk->y_move_period_time*walk->ssp_time_start_left,walk->y_move_amplitude,walk->y_move_amplitude_shift);
    z_move_l=wsin(walk->ssp_time_start_left,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_left,walk->z_move_amplitude,walk->z_move_amplitude_shift);
    c_move_l=wsin(walk->ssp_time_start_left,walk->a_move_period_time,walk->a_move_phase_shift+2.0*3.14159/walk->a_move_period_time*walk->ssp_time_start_left,walk->a_move_amplitude,walk->a_move_amplitude_shift);
    x_move_r=wsin(walk->ssp_time_start_left,walk->x_move_period_time,walk->x_move_phase_shift+2.0*3.14159/walk->x_move_period_time*walk->ssp_time_start_left,-walk->x_move_amplitude,-walk->x_move_amplitude_shift);
    y_move_r=wsin(walk->ssp_time_start_left,walk->y_move_period_time,walk->y_move_phase_shift+2.0*3.14159/walk->y_move_period_time*walk->ssp_time_start_left,-walk->y_move_amplitude,-walk->y_move_amplitude_shift);
    z_move_r=wsin(walk->ssp_time_start_right,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_right,walk->z_move_amplitude,walk->z_move_amplitude_shift);
    c_move_r=wsin(walk->ssp_time_start_left,walk->a_move_period_time,walk->a_move_phase_shift+2.0*3.14159/walk->a_move_period_time*walk->ssp_time_start_left,-walk->a_move_amplitude,-walk->a_move_amplitude_shift);
    pelvis_offset_l=0;
    pelvis_offset_r=0;
  }
  else if(walk->time<=walk->ssp_time_end_left)
  {
    x_move_l=wsin(walk->time,walk->x_move_period_time,walk->x_move_phase_shift+2.0*3.14159/walk->x_move_period_time*walk->ssp_time_start_left,walk->x_move_amplitude,walk->x_move_amplitude_shift);
    y_move_l=wsin(walk->time,walk->y_move_period_time,walk->y_move_phase_shift+2.0*3.14159/walk->y_move_period_time*walk->ssp_time_start_left,walk->y_move_amplitude,walk->y_move_amplitude_shift);
    z_move_l=wsin(walk->time,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_left,walk->z_move_amplitude,walk->z_move_amplitude_shift);
    c_move_l=wsin(walk->time,walk->a_move_period_time,walk->a_move_phase_shift+2.0*3.14159/walk->a_move_period_time*walk->ssp_time_start_left,walk->a_move_amplitude,walk->a_move_amplitude_shift);
    x_move_r=wsin(walk->time,walk->x_move_period_time,walk->x_move_phase_shift+2.0*3.14159/walk->x_move_period_time*walk->ssp_time_start_left,-walk->x_move_amplitude,-walk->x_move_amplitude_shift);
    y_move_r=wsin(walk->time,walk->y_move_period_time,walk->y_move_phase_shift+2.0*3.14159/walk->y_move_period_time*walk->ssp_time_start_left,-walk->y_move_amplitude,-walk->y_move_amplitude_shift);
    z_move_r=wsin(walk->ssp_time_start_right,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_right,walk->z_move_amplitude,walk->z_move_amplitude_shift);
    c_move_r=wsin(walk->time,walk->a_move_period_time,walk->a_move_phase_shift+2.0*3.14159/walk->a_move_period_time*walk->ssp_time_start_left,-walk->a_move_amplitude,-walk->a_move_amplitude_shift);
    pelvis_offset_l=wsin(walk->time,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_left,walk->pelvis_swing/2.0,walk->pelvis_swing/2.0);
    pelvis_offset_r=wsin(walk->time,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_left,-walk->pelvis_offset/2.0,-walk->pelvis_offset/2.0);
  }
  else if(walk->time<=walk->ssp_time_start_right)
  {
    x_move_l=wsin(walk->ssp_time_end_left,walk->x_move_period_time,walk->x_move_phase_shift+2.0*3.14159/walk->x_move_period_time*walk->ssp_time_start_left,walk->x_move_amplitude,walk->x_move_amplitude_shift);
    y_move_l=wsin(walk->ssp_time_end_left,walk->y_move_period_time,walk->y_move_phase_shift+2.0*3.14159/walk->y_move_period_time*walk->ssp_time_start_left,walk->y_move_amplitude,walk->y_move_amplitude_shift);
    z_move_l=wsin(walk->ssp_time_end_left,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_left,walk->z_move_amplitude,walk->z_move_amplitude_shift);
    c_move_l=wsin(walk->ssp_time_end_left,walk->a_move_period_time,walk->a_move_phase_shift+2.0*3.14159/walk->a_move_period_time*walk->ssp_time_start_left,walk->a_move_amplitude,walk->a_move_amplitude_shift);
    x_move_r=wsin(walk->ssp_time_end_left,walk->x_move_period_time,walk->x_move_phase_shift+2.0*3.14159/walk->x_move_period_time*walk->ssp_time_start_left,-walk->x_move_amplitude,-walk->x_move_amplitude_shift);
    y_move_r=wsin(walk->ssp_time_end_left,walk->y_move_period_time,walk->y_move_phase_shift+2.0*3.14159/walk->y_move_period_time*walk->ssp_time_start_left,-walk->y_move_amplitude,-walk->y_move_amplitude_shift);
    z_move_r=wsin(walk->ssp_time_start_right,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_right,walk->z_move_amplitude,walk->z_move_amplitude_shift);
    c_move_r=wsin(walk->ssp_time_end_left,walk->a_move_period_time,walk->a_move_phase_shift+2.0*3.14159/walk->a_move_period_time*walk->ssp_time_start_left,-walk->a_move_amplitude,-walk->a_move_amplitude_shift);
    pelvis_offset_l=0.0;
    pelvis_offset_r=0.0;
  }
  else if(walk->time<=walk->ssp_time_end_right)
  {
    x_move_l=wsin(walk->time,walk->x_move_period_time,walk->x_move_phase_shift+2.0*3.14159/walk->x_move_period_time*walk->ssp_time_start_right+3.14159,walk->x_move_amplitude,walk->x_move_amplitude_shift);
    y_move_l=wsin(walk->time,walk->y_move_period_time,walk->y_move_phase_shift+2.0*3.14159/walk->y_move_period_time*walk->ssp_time_start_right+3.14159,walk->y_move_amplitude,walk->y_move_amplitude_shift);
    z_move_l=wsin(walk->ssp_time_end_left,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_left,walk->z_move_amplitude,walk->z_move_amplitude_shift);
    c_move_l=wsin(walk->time,walk->a_move_period_time,walk->a_move_phase_shift+2.0*3.14159/walk->a_move_period_time*walk->ssp_time_start_right+3.14159,walk->a_move_amplitude,walk->a_move_amplitude_shift);
    x_move_r=wsin(walk->time,walk->x_move_period_time,walk->x_move_phase_shift+2.0*3.14159/walk->x_move_period_time*walk->ssp_time_start_right+3.14159,-walk->x_move_amplitude,-walk->x_move_amplitude_shift);
    y_move_r=wsin(walk->time,walk->y_move_period_time,walk->y_move_phase_shift+2.0*3.14159/walk->y_move_period_time*walk->ssp_time_start_right+3.14159,-walk->y_move_amplitude,-walk->y_move_amplitude_shift);
    z_move_r=wsin(walk->time,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_right,walk->z_move_amplitude,walk->z_move_amplitude_shift);
    c_move_r=wsin(walk->time,walk->a_move_period_time,walk->a_move_phase_shift+2.0*3.14159/walk->a_move_period_time*walk->ssp_time_start_right+3.14159,-walk->a_move_amplitude,-walk->a_move_amplitude_shift);
    pelvis_offset_l=wsin(walk->time,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_right,walk->pelvis_offset/2,walk->pelvis_offset/2);
    pelvis_offset_r=wsin(walk->time,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_right,-walk->pelvis_swing/2,-walk->pelvis_swing/2);
  }
  else
  {
    x_move_l=wsin(walk->ssp_time_end_right,walk->x_move_period_time,walk->x_move_phase_shift+2.0*3.14159/walk->x_move_period_time*walk->ssp_time_start_right+3.14159,walk->x_move_amplitude,walk->x_move_amplitude_shift);
    y_move_l=wsin(walk->ssp_time_end_right,walk->y_move_period_time,walk->y_move_phase_shift+2.0*3.14159/walk->y_move_period_time*walk->ssp_time_start_right+3.14159,walk->y_move_amplitude,walk->y_move_amplitude_shift);
    z_move_l=wsin(walk->ssp_time_end_left,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_left,walk->z_move_amplitude,walk->z_move_amplitude_shift);
    c_move_l=wsin(walk->ssp_time_end_right,walk->a_move_period_time,walk->a_move_phase_shift+2.0*3.14159/walk->a_move_period_time*walk->ssp_time_start_right+3.14159,walk->a_move_amplitude,walk->a_move_amplitude_shift);
    x_move_r=wsin(walk->ssp_time_end_right,walk->x_move_period_time,walk->x_move_phase_shift+2.0*3.14159/walk->x_move_period_time*walk->ssp_time_start_right+3.14159,-walk->x_move_amplitude,-walk->x_move_amplitude_shift);
    y_move_r=wsin(walk->ssp_time_end_right,walk->y_move_period_time,walk->y_move_phase_shift+2.0*3.14159/walk->y_move_period_time*walk->ssp_time_start_right+3.14159,-walk->y_move_amplitude,-walk->y_move_amplitude_shift);
    z_move_r=wsin(walk->ssp_time_end_right,walk->z_move_period_time,walk->z_move_phase_shift+2.0*3.14159/walk->z_move_period_time*walk->ssp_time_start_right,walk->z_move_amplitude,walk->z_move_amplitude_shift);
    c_move_r=wsin(walk->ssp_time_end_right,walk->a_move_period_time,walk->a_move_phase_shift+2.0*3.14159/walk->a_move_period_time*walk->ssp_time_start_right+3.14159,-walk->a_move_amplitude,-walk->a_move_amplitude_shift);
    pelvis_offset_l=0.0;
    pelvis_offset_r=0.0;
  }
  a_move_l=0.0;
  b_move_l=0.0;
  a_move_r=0.0;
  b_move_r=0.0;

  ep[0]=x_swap+x_move_r+walk->x_offset;
  ep[1]=y_swap+y_move_r-walk->y_offset / 2.0;
  ep[2]=z_swap+z_move_r+walk->z_offset;
  ep[3]=a_swap+a_move_r-walk->r_offset / 2.0;
  ep[4]=b_swap+b_move_r+walk->p_offset;
  ep[5]=c_swap+c_move_r-walk->a_offset / 2.0;
  ep[6]=x_swap+x_move_l+walk->x_offset;
  ep[7]=y_swap+y_move_l+walk->y_offset / 2.0;
  ep[8]=z_swap+z_move_l+walk->z_offset;
  ep[9]=a_swap+a_move_l+walk->r_offset / 2.0;
  ep[10]=b_swap+b_move_l+walk->p_offset;
  ep[11]=c_swap+c_move_l+walk->a_offset / 2.0;

  // Compute body swing
/*  if(time<=ssp_time_end_left)
  {
    m_Body_Swing_Y=-ep[7];
    m_Body_Swing_Z=ep[8];
  }
  else
  {
    m_Body_Swing_Y=-ep[1];
    m_Body_Swing_Z=ep[2];
  }
  m_Body_Swing_Z-=LEG_LENGTH;*/

  // Compute arm swing
  if(walk->x_move_amplitude==0)
  {
    angle[12]=0; // Right
    angle[13]=0; // Left
  }
  else
  {
    angle[12]=wsin(walk->time,walk->period_time,0.0,-walk->x_move_amplitude*walk->arm_swing_gain,0.0);
    angle[13]=wsin(walk->time,walk->period_time,0.0,walk->x_move_amplitude*walk->arm_swing_gain,0.0);
  }

  if(walk->memory->data[walk->ram_base_address+WALK_CONTROL]&WALK_RUNNING)
  {
    walk->time+=walk->walk_period;
    if(walk->time>=walk->period_time)
      walk->time=0;
  }

  // Compute angles with the inverse kinematics
  if(walk->leg_ik_function!=0x00000000)
  {
    if((walk->leg_ik_function(&angle[0],ep[0],ep[1],ep[2],ep[3],ep[4],ep[5])!=1) || 
       (walk->leg_ik_function(&angle[6],ep[6],ep[7],ep[8],ep[9],ep[10],ep[11])!=1))
      return;// Do not use angles
  }
  else
    return;

  // Compute motor value
  if(mmanager_get_module(walk->mmodule.manager,walk->right_hip_yaw_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->right_hip_yaw_servo_id].target_angle=((-180.0*angle[0])/3.14159)*65536.0;
  if(mmanager_get_module(walk->mmodule.manager,walk->right_hip_roll_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->right_hip_roll_servo_id].target_angle=((-180.0*(angle[1]+pelvis_offset_r))/3.14159)*65536.0;
  if(mmanager_get_module(walk->mmodule.manager,walk->right_hip_pitch_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->right_hip_pitch_servo_id].target_angle=((180.0*angle[2])/3.14159-walk->hip_pitch_offset)*65536.0;
  if(mmanager_get_module(walk->mmodule.manager,walk->right_knee_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->right_knee_servo_id].target_angle=((180.0*angle[3])/3.14159)*65536.0;
  if(mmanager_get_module(walk->mmodule.manager,walk->right_ankle_pitch_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->right_ankle_pitch_servo_id].target_angle=((-180.0*angle[4])/3.14159)*65536.0;
  if(mmanager_get_module(walk->mmodule.manager,walk->right_ankle_roll_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->right_ankle_roll_servo_id].target_angle=((180.0*angle[5])/3.14159)*65536.0;
  if(mmanager_get_module(walk->mmodule.manager,walk->left_hip_yaw_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->left_hip_yaw_servo_id].target_angle=((-180.0*angle[6])/3.14159)*65536.0;
  if(mmanager_get_module(walk->mmodule.manager,walk->left_hip_roll_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->left_hip_roll_servo_id].target_angle=((-180.0*(angle[7]+pelvis_offset_l))/3.14159)*65536.0;
  if(mmanager_get_module(walk->mmodule.manager,walk->left_hip_pitch_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->left_hip_pitch_servo_id].target_angle=((-180.0*angle[8])/3.14159+walk->hip_pitch_offset)*65536.0;
  if(mmanager_get_module(walk->mmodule.manager,walk->left_knee_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->left_knee_servo_id].target_angle=((-180.0*angle[9])/3.14159)*65536.0;
  if(mmanager_get_module(walk->mmodule.manager,walk->left_ankle_pitch_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->left_ankle_pitch_servo_id].target_angle=((180.0*angle[10])/3.14159)*65536.0;
  if(mmanager_get_module(walk->mmodule.manager,walk->left_ankle_roll_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->left_ankle_roll_servo_id].target_angle=((180.0*angle[11])/3.14159)*65536.0;
  if(mmanager_get_module(walk->mmodule.manager,walk->right_shoulder_pitch_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->right_shoulder_pitch_servo_id].target_angle=(angle[12]-48.345)*65536.0;
  if(mmanager_get_module(walk->mmodule.manager,walk->left_shoulder_pitch_servo_id)==MM_WALK)
    walk->mmodule.manager->servo_values[walk->left_shoulder_pitch_servo_id].target_angle=(-angle[13]+41.313)*65536.0;
}

TMotionModule *walk_get_module(TWalkMModule *walk)
{
  return &walk->mmodule;
}

void walk_set_period(void *module,unsigned short int period_ms)
{
  TWalkMModule *walk=(TWalkMModule *)module;

  walk->walk_period=((float)period_ms)/1000000.0;
}

void walk_set_module(void *module,unsigned char servo_id)
{

}

// public functions
unsigned char walk_init(TWalkMModule *walk,TMemory *memory,unsigned short int ram_base_address,unsigned short int eeprom_base_address)
{
  // initialize the internal motion variables
  walk->aim_on=0x00;
  walk->x_swap_phase_shift=3.14159;
  walk->x_swap_amplitude_shift=0.0;
  walk->x_move_phase_shift=3.14159/2.0;
  walk->x_move_amplitude=0.0;
  walk->x_move_amplitude_shift=0.0;
  walk->y_swap_phase_shift=0.0;
  walk->y_swap_amplitude_shift=0.0;
  walk->y_move_phase_shift=3.14159/2.0;
  walk->z_swap_phase_shift=3.14159*3.0/2.0;
  walk->z_move_phase_shift=3.14159/2.0;
  walk->a_move_phase_shift=3.14159/2.0;

  // initialize internal control variables
  walk->time=0;
  walk->running=0x00;
  walk->leg_ik_function=0x00000000;

//  update_param_time(walk);
//  update_param_move(walk);

//  walk_pre_process(walk);

  /* initialize the motion module */
  mmodule_init(&walk->mmodule);
  walk->mmodule.id=MM_WALK;
  walk->mmodule.set_period=walk_set_period;
  walk->mmodule.set_module=walk_set_module;
  walk->mmodule.pre_process=walk_pre_process;
  walk->mmodule.data=walk;

  mem_module_init(&walk->mem_module);
  walk->mem_module.data=walk;
  walk->mem_module.write_cmd=(void(*)(void *,unsigned short int, unsigned short int,unsigned char *))walk_write_cmd;
  walk->mem_module.read_cmd=(void(*)(void *,unsigned short int, unsigned short int,unsigned char *))walk_read_cmd;
  if(!mem_module_add_ram_segment(&walk->mem_module,ram_base_address,RAM_WALK_LENGTH))
    return 0x00;
  if(!mem_module_add_eeprom_segment(&walk->mem_module,eeprom_base_address,EEPROM_WALK_LENGTH))
    return 0x00;
  if(!mem_add_module(memory,&walk->mem_module))
    return 0x00;
  walk->ram_base_address=ram_base_address;
  walk->eeprom_base_address=eeprom_base_address;
  walk->memory=memory;

  return 0x01;
}

void walk_start(TWalkMModule *walk)
{
  walk->memory->data[walk->ram_base_address+WALK_CONTROL]|=WALK_RUNNING;
  walk->running=0x01;
}

void walk_stop(TWalkMModule *walk)
{
  walk->running=0x00;
}

unsigned char is_walk(TWalkMModule *walk)
{
  if(walk->memory->data[walk->ram_base_address+WALK_CONTROL]&WALK_RUNNING)
    return 0x01;
  else 
    return 0x00;
}


