#include "ram.h"

unsigned char ram_init(TMemory *memory)
{
  mem_initialize_data(memory);

  return 0x01;
}

unsigned char ram_read_table(TMemory *memory,unsigned short int address, unsigned short int length,unsigned char *data)
{
  unsigned short int i;

  if((address+length)<=(RAM_SIZE-1))
  {
    for(i=0;i<length;i++)
      data[i]=memory->data[address+i];
    return RAM_SUCCESS;
  }
  else
    return RAM_BAD_ADDRESS;
}

unsigned char ram_set_bit(TMemory *memory,unsigned short int address, unsigned char bit)
{
  if(bit>=0 && bit<8)
  {
    memory->data[address]|=(0x01<<bit);
    return RAM_SUCCESS;
  }
  else
    return RAM_BAD_BIT;
}

unsigned char ram_clear_bit(TMemory *memory,unsigned short int address, unsigned char bit)
{
  if(bit>=0 && bit<8)
  {
    memory->data[address]&=(~(0x01<<bit));
    return RAM_SUCCESS;
  }
  else
    return RAM_BAD_BIT;
}

unsigned char ram_write_byte(TMemory *memory,unsigned short int address, unsigned char data)
{
  memory->data[address]=data;

  return RAM_SUCCESS;
}

unsigned char ram_write_word(TMemory *memory,unsigned short int address, unsigned short int data)
{
  if(address < (RAM_SIZE-1))
  {
    memory->data[address]=data%256;
    memory->data[address+1]=data/256;

    return RAM_SUCCESS;
  }
  else
    return RAM_BAD_ADDRESS;  
}

unsigned char ram_write_table(TMemory *memory,unsigned short int address, unsigned short int length,unsigned char *data)
{
  unsigned short int i;

  if((address+length)<RAM_SIZE)
  {
    for(i=0;i<length;i++)
      memory->data[address+i]=data[i];
    return RAM_SUCCESS;
  }
  else
    return RAM_BAD_ADDRESS;
}

unsigned char ram_in_window(unsigned short int start_reg,unsigned short int reg_length,unsigned short int start_address,unsigned short int address_length)
{
  unsigned short int end_reg=start_reg+reg_length-1;
  unsigned short int end_address=start_address+address_length-1;

  if((start_reg>=start_address && start_reg<=end_address) || (end_reg>=start_address && end_reg<=end_address) ||
     (start_address>=start_reg && start_address<=end_reg))
    return 0x01;
  else
    return 0x00;
}
