#include "frame_buffer_control.h"
#include "patterns.h"
#include "waveform_patterns.h"
#include "image_patterns.h"
#include "motion_patterns.h"
#include "ram.h"

/* private functions */
void frame_buffer_control_write_cmd(void *module,unsigned short int address,unsigned short int length,unsigned char *data)
{
  TFBControl *control=(TFBControl *)module;
  static TLEDArea ptrn_area={0};
  static unsigned char ptrn_data[FBC_MAX_DATA_PATTERN_LEN]={0},data_len=0,pattern_id=0,pattern_index=0,buffer_id=0;
  unsigned char *data_ptr;
  unsigned short int new_value,ram_offset,eeprom_offset,i;
  pattern_func_t function=0x00000000;

  ram_offset=address-control->ram_base_address;
  eeprom_offset=address-control->eeprom_base_address;
  if(ram_in_window(control->ram_base_address+FRAME_BUFFER_CONTROL_OFFSET,1,address,length))
  {
    if(data[FRAME_BUFFER_CONTROL_OFFSET-ram_offset]&FRAME_BUFFER_CONTROL_LOAD)// add the new pattern
    {
      switch(pattern_id&PATTERN_GROUP)
      {
        case WAVEFORM_GROUP:
          function=(pattern_func_t)wf_general;
          // initialize static data
          ptrn_data[WF_CURRENT_PERIOD_OFFSET]=0x00;
          ptrn_data[WF_CURRENT_PERIOD_OFFSET+1]=0x00;
          data_len+=2;
          break;
        case MOTION_GROUP:
          function=(pattern_func_t)mtn_general;
          ptrn_data[MTN_CURRENT_COL_OFFSET]=0x00;
          ptrn_data[MTN_CURRENT_COL_OFFSET+1]=0x00;
          ptrn_data[MTN_CURRENT_ROW_OFFSET]=0x00;
          ptrn_data[MTN_CURRENT_ROW_OFFSET+1]=0x00;
          ptrn_data[MTN_CURRENT_TIME_OFFSET]=0x00;
          ptrn_data[MTN_CURRENT_TIME_OFFSET+1]=0x00;
          data_len+=6;
          break;
        case IMAGE_GROUP:
          function=(pattern_func_t)img_general;
          break;
        default: break;
      }
      frame_buffer_control_add_pattern(control,function,&ptrn_area,ptrn_data,data_len,buffer_id);
      /* clear the internal data for the next pattern */
      data_len=0;
      ptrn_area.min_row=0;
      ptrn_area.max_row=0;
      ptrn_area.min_col=0;
      ptrn_area.max_col=0;
      pattern_id=0;
      buffer_id=0;
      for(i=0;i<FBC_MAX_DATA_PATTERN_LEN;i++)
        ptrn_data[i]=0;
    }
    else if(data[FRAME_BUFFER_CONTROL_OFFSET-ram_offset]&FRAME_BUFFER_CONTROL_REMOVE)
      frame_buffer_control_remove_pattern(control,pattern_index);
    else if(data[FRAME_BUFFER_CONTROL_OFFSET-ram_offset]&FRAME_BUFFER_CONTROL_CLEAR)
      frame_buffer_control_clear_patterns(control);
    else if(data[FRAME_BUFFER_CONTROL_OFFSET-ram_offset]&FRAME_BUFFER_CONTROL_START)
      frame_buffer_control_start(control);
    else if(data[FRAME_BUFFER_CONTROL_OFFSET-ram_offset]&FRAME_BUFFER_CONTROL_STOP)
      frame_buffer_control_stop(control);
  }
  if(ram_in_window(control->ram_base_address+FRAME_BUFFER_CONTROL_PATTERN_INDEX_OFFSET,1,address,length))
    pattern_index=data[FRAME_BUFFER_CONTROL_PATTERN_INDEX_OFFSET-ram_offset];
  if(ram_in_window(control->ram_base_address+FRAME_BUFFER_CONTROL_PATTERN_ID_OFFSET,1,address,length))
    pattern_id=data[FRAME_BUFFER_CONTROL_PATTERN_ID_OFFSET-ram_offset];
  if(ram_in_window(control->ram_base_address+FRAME_BUFFER_CONTROL_MIN_ROW_OFFSET,2,address,length))
  {
    data_ptr=(unsigned char *)&ptrn_area.min_row;
    if(ram_in_range(control->ram_base_address+FRAME_BUFFER_CONTROL_MIN_ROW_OFFSET,address,length))
      data_ptr[0]=data[FRAME_BUFFER_CONTROL_MIN_ROW_OFFSET-ram_offset];
    if(ram_in_range(control->ram_base_address+FRAME_BUFFER_CONTROL_MIN_ROW_OFFSET+1,address,length))
      data_ptr[1]=data[FRAME_BUFFER_CONTROL_MIN_ROW_OFFSET+1-ram_offset];
  }
  if(ram_in_window(control->ram_base_address+FRAME_BUFFER_CONTROL_MAX_ROW_OFFSET,2,address,length))
  {
    data_ptr=(unsigned char *)&ptrn_area.max_row;
    if(ram_in_range(control->ram_base_address+FRAME_BUFFER_CONTROL_MAX_ROW_OFFSET,address,length))
      data_ptr[0]=data[FRAME_BUFFER_CONTROL_MAX_ROW_OFFSET-ram_offset];
    if(ram_in_range(control->ram_base_address+FRAME_BUFFER_CONTROL_MAX_ROW_OFFSET+1,address,length))
      data_ptr[1]=data[FRAME_BUFFER_CONTROL_MAX_ROW_OFFSET+1-ram_offset];
  }
  if(ram_in_window(control->ram_base_address+FRAME_BUFFER_CONTROL_MIN_COL_OFFSET,2,address,length))
  {
    data_ptr=(unsigned char *)&ptrn_area.min_col;
    if(ram_in_range(control->ram_base_address+FRAME_BUFFER_CONTROL_MIN_COL_OFFSET,address,length))
      data_ptr[0]=data[FRAME_BUFFER_CONTROL_MIN_COL_OFFSET-ram_offset];
    if(ram_in_range(control->ram_base_address+FRAME_BUFFER_CONTROL_MIN_COL_OFFSET+1,address,length))
      data_ptr[1]=data[FRAME_BUFFER_CONTROL_MIN_COL_OFFSET+1-ram_offset];
  }
  if(ram_in_window(control->ram_base_address+FRAME_BUFFER_CONTROL_MAX_COL_OFFSET,2,address,length))
  {
    data_ptr=(unsigned char *)&ptrn_area.max_col;
    if(ram_in_range(control->ram_base_address+FRAME_BUFFER_CONTROL_MAX_COL_OFFSET,address,length))
      data_ptr[0]=data[FRAME_BUFFER_CONTROL_MAX_COL_OFFSET-ram_offset];
    if(ram_in_range(control->ram_base_address+FRAME_BUFFER_CONTROL_MAX_COL_OFFSET+1,address,length))
      data_ptr[1]=data[FRAME_BUFFER_CONTROL_MAX_COL_OFFSET+1-ram_offset];
  }
  if(ram_in_window(control->ram_base_address+FRAME_BUFFER_CONTROL_BUFFER_ID_OFFSET,1,address,length))
    buffer_id=data[FRAME_BUFFER_CONTROL_BUFFER_ID_OFFSET-ram_offset];
  if(ram_in_window(control->ram_base_address+FRAME_BUFFER_CONTROL_DATA_OFFSET,FBC_MAX_DATA_PATTERN_LEN,address,length))
  {
    for(i=0;i<length-(FRAME_BUFFER_CONTROL_DATA_OFFSET-ram_offset) && i<FBC_MAX_DATA_PATTERN_LEN;i++)
    {
      ptrn_data[i]=data[FRAME_BUFFER_CONTROL_DATA_OFFSET-ram_offset+i];
      data_len++;
    }
  }
  if(ram_in_window(control->eeprom_base_address+FRAME_BUFFER_CONTROL_PERIOD_OFFSET,2,address,length))
  {
    new_value=control->period_ms;
    data_ptr=(unsigned char *)&new_value;
    if(ram_in_range(control->eeprom_base_address+FRAME_BUFFER_CONTROL_PERIOD_OFFSET,address,length))
      data_ptr[0]=data[FRAME_BUFFER_CONTROL_PERIOD_OFFSET-eeprom_offset];
    if(ram_in_range(control->eeprom_base_address+FRAME_BUFFER_CONTROL_PERIOD_OFFSET+1,address,length))
      data_ptr[1]=data[FRAME_BUFFER_CONTROL_PERIOD_OFFSET+1-eeprom_offset];
    frame_buffer_control_set_period(control,new_value);
  }
  if(ram_in_window(control->eeprom_base_address+FRAME_BUFFER_CONTROL_NUM_ROWS_OFFSET,2,address,length))
  {
    new_value=control->frame_buffer.num_rows;
    data_ptr=(unsigned char *)&new_value;
    if(ram_in_range(control->eeprom_base_address+FRAME_BUFFER_CONTROL_NUM_ROWS_OFFSET,address,length))
      data_ptr[0]=data[FRAME_BUFFER_CONTROL_NUM_ROWS_OFFSET-eeprom_offset];
    if(ram_in_range(control->eeprom_base_address+FRAME_BUFFER_CONTROL_NUM_ROWS_OFFSET+1,address,length))
      data_ptr[1]=data[FRAME_BUFFER_CONTROL_NUM_ROWS_OFFSET+1-eeprom_offset];
    frame_buffer_set_num_rows(&control->frame_buffer,new_value);
  }
  if(ram_in_window(control->eeprom_base_address+FRAME_BUFFER_CONTROL_NUM_PIXELS_OFFSET,2,address,length))
  {
    new_value=control->frame_buffer.num_pixels_per_row;
    data_ptr=(unsigned char *)&new_value;
    if(ram_in_range(control->eeprom_base_address+FRAME_BUFFER_CONTROL_NUM_PIXELS_OFFSET,address,length))
      data_ptr[0]=data[FRAME_BUFFER_CONTROL_NUM_PIXELS_OFFSET-eeprom_offset];
    if(ram_in_range(control->eeprom_base_address+FRAME_BUFFER_CONTROL_NUM_PIXELS_OFFSET+1,address,length))
      data_ptr[1]=data[FRAME_BUFFER_CONTROL_NUM_PIXELS_OFFSET+1-eeprom_offset];
    frame_buffer_set_pixels_per_row(&control->frame_buffer,new_value);
  }
  if(ram_in_window(control->eeprom_base_address+FRAME_BUFFER_CONTROL_NUM_BUFFERS_OFFSET,2,address,length))
  {
    new_value=control->frame_buffer.num_buffers;
    data_ptr=(unsigned char *)&new_value;
    if(ram_in_range(control->eeprom_base_address+FRAME_BUFFER_CONTROL_NUM_BUFFERS_OFFSET,address,length))
      data_ptr[0]=data[FRAME_BUFFER_CONTROL_NUM_BUFFERS_OFFSET-eeprom_offset];
    if(ram_in_range(control->eeprom_base_address+FRAME_BUFFER_CONTROL_NUM_BUFFERS_OFFSET+1,address,length))
      data_ptr[1]=data[FRAME_BUFFER_CONTROL_NUM_BUFFERS_OFFSET+1-eeprom_offset];
    frame_buffer_set_num_buffers(&control->frame_buffer,new_value);
  }
  if(ram_in_window(control->ram_base_address+FRAME_BUFFER_CONTROL_IMG_DATA_OFFSET,IMG_MAX_NUM_IMAGES*IMG_MAX_IMAGE_WIDTH*IMG_MAX_IMAGE_HEIGHT*3,address,length))
  {
    for(i=0;i<length;i++)
      ((unsigned char *)img_pattern_data)[address-control->ram_base_address-FRAME_BUFFER_CONTROL_IMG_DATA_OFFSET+i]=data[i];
  }
}

void frame_buffer_control_read_cmd(void *module,unsigned short int address,unsigned short int length,unsigned char *data)
{
  TFBControl *control=(TFBControl *)module;

  ram_read_table(control->memory,address,length,data);
}

void frame_buffer_control_scheduler(void *data)
{
  TFBControl *control=(TFBControl *)data;
  unsigned char i;

  frame_buffer_control_update(control);
  for(i=0;i<control->num_copy_functions;i++)
    control->copy_functions[i](control->copy_data[i]);
}

/* public functions */
unsigned char frame_buffer_control_init(TFBControl *control,TMemory *memory,TScheduler *scheduler,sched_channel_t ch,unsigned short int ram_base_address,unsigned short int eeprom_base_address)
{
  // initialize internal variables
  frame_buffer_control_clear_patterns(control);
  frame_buffer_control_clear_copy_functions(control);
  frame_buffer_init(&control->frame_buffer,memory,ram_base_address,eeprom_base_address);
  control->ram_base_address=ram_base_address;
  control->eeprom_base_address=eeprom_base_address;

  /* assigna a scheduler channel */
  scheduler_set_channel(scheduler,ch,frame_buffer_control_scheduler,DEFAULT_FRAME_BUFFER_CONTROL_PERIOD,control);
  control->scheduler=scheduler;
  control->sch_channel=ch;

  img_patterns_init(memory,ram_base_address);

  /* initialize memory module */
  mem_module_init(&control->mem_module);
  control->mem_module.write_cmd=frame_buffer_control_write_cmd;
  control->mem_module.read_cmd=frame_buffer_control_read_cmd;
  control->mem_module.data=control;
  control->memory=memory;
  if(!mem_module_add_ram_segment(&control->mem_module,ram_base_address,RAM_FRAME_BUFFER_CONTROL_LENGTH))
    return 0x00;
  if(!mem_module_add_eeprom_segment(&control->mem_module,eeprom_base_address,EEPROM_FRAME_BUFFER_CONTROL_LENGTH))
    return 0x00;
  if(!mem_add_module(memory,&control->mem_module))
    return 0x00;

  return 0x01;
}

void frame_buffer_control_start(TFBControl *control)
{
  scheduler_enable_channel(control->scheduler,control->sch_channel);
  control->memory->data[control->ram_base_address+FRAME_BUFFER_CONTROL_OFFSET]|=FRAME_BUFFER_CONTROL_RUNNING;
}

void frame_buffer_control_stop(TFBControl *control)
{
  scheduler_disable_channel(control->scheduler,control->sch_channel);
  control->memory->data[control->ram_base_address+FRAME_BUFFER_CONTROL_OFFSET]&=(~FRAME_BUFFER_CONTROL_RUNNING);
}

void frame_buffer_control_set_period(TFBControl *control,unsigned short int period_ms)
{
  scheduler_change_period(control->scheduler,control->sch_channel,period_ms);
  control->memory->data[control->eeprom_base_address+FRAME_BUFFER_CONTROL_PERIOD_OFFSET]=period_ms&0x00FF;
  control->memory->data[control->eeprom_base_address+FRAME_BUFFER_CONTROL_PERIOD_OFFSET+1]=(period_ms&0xFF00)>>8;
  control->period_ms=period_ms;
}

unsigned short int frame_buffer_control_get_period(TFBControl *control)
{
  return control->period_ms;
}

// pattern functions
unsigned char frame_buffer_control_add_pattern(TFBControl *control,pattern_func_t function,TLEDArea *area,void *data,unsigned char data_length,unsigned char buffer_id)
{
  unsigned short int num_rows,num_cols;
  unsigned char i,j;

  num_rows=frame_buffer_get_num_rows(&control->frame_buffer);
  num_cols=frame_buffer_get_pixels_per_row(&control->frame_buffer); 
  if(area->min_row>num_rows || area->max_row>num_rows || area->min_row>area->max_row)
  {
    control->memory->data[control->ram_base_address+FRAME_BUFFER_CONTROL_PATTERN_INDEX_OFFSET]=0xFF;
    return 0xFF;
  }
  if(area->min_col>num_cols || area->max_col>num_cols || area->min_col>area->max_col)
  {
    control->memory->data[control->ram_base_address+FRAME_BUFFER_CONTROL_PATTERN_INDEX_OFFSET]=0xFF;
    return 0xFF;
  }
  if(data_length>FBC_MAX_DATA_PATTERN_LEN)
  {
    control->memory->data[control->ram_base_address+FRAME_BUFFER_CONTROL_PATTERN_INDEX_OFFSET]=0xFF;
    return 0xFF;
  }
  if(buffer_id>=frame_buffer_get_num_buffers(&control->frame_buffer))
  {
    control->memory->data[control->ram_base_address+FRAME_BUFFER_CONTROL_PATTERN_INDEX_OFFSET]=0xFF;
    return 0xFF;
  }

  for(j=0;j<FBC_MAX_PATTERN_FUNC;j++)
  {
    // find the first empty pattern
    if(control->pat_functions[j]==0x00000000)
    {
      control->pat_area[j].min_row=area->min_row;
      control->pat_area[j].max_row=area->max_row;
      control->pat_area[j].min_col=area->min_col;
      control->pat_area[j].max_col=area->max_col;
      control->pat_buffer_id[j]=buffer_id;
      for(i=0;i<data_length;i++)
        control->pat_parameters[j][i]=((unsigned char *)data)[i];
      control->pat_functions[j]=function;
      control->num_patterns++;
      control->memory->data[control->ram_base_address+FRAME_BUFFER_CONTROL_NUM_PATTERN_OFFSET]=control->num_patterns;
      control->memory->data[control->ram_base_address+FRAME_BUFFER_CONTROL_PATTERN_INDEX_OFFSET]=j;
      return j;
    }
  }
  // the pattern could not be added
  control->memory->data[control->ram_base_address+FRAME_BUFFER_CONTROL_PATTERN_INDEX_OFFSET]=0xFF;
  return 0xFF;
}

void frame_buffer_control_remove_pattern(TFBControl *control,unsigned char index)
{
  unsigned char i;

  if(control->num_patterns>0 && index<FBC_MAX_PATTERN_FUNC)
  {
    control->pat_functions[index]=0x00000000;
    control->pat_area[index].min_row=0;
    control->pat_area[index].max_row=0;
    control->pat_area[index].min_col=0;
    control->pat_area[index].max_col=0;
    control->pat_buffer_id[index]=-1;
    for(i=0;i<FBC_MAX_DATA_PATTERN_LEN;i++)
      control->pat_parameters[index][i]=0;
    control->num_patterns--; 
  } 
}

void frame_buffer_control_clear_patterns(TFBControl *control)
{
  unsigned char i,j;

  control->num_patterns=0;
  for(i=0;i<FBC_MAX_PATTERN_FUNC;i++)
  {
    control->pat_functions[i]=0x00000000;
    control->pat_area[i].min_row=0;
    control->pat_area[i].max_row=0;
    control->pat_area[i].min_col=0;
    control->pat_area[i].max_col=0;
    control->pat_buffer_id[i]=-1;
    for(j=0;j<FBC_MAX_DATA_PATTERN_LEN;j++)
      control->pat_parameters[i][j]=0;
  }
}

unsigned char frame_buffer_control_get_num_patterns(TFBControl *control)
{
  return control->num_patterns;
}

unsigned char *frame_buffer_control_get_parameters(TFBControl *control,unsigned char index)
{
  if(index>=0 && index<FBC_MAX_PATTERN_FUNC)
    return (unsigned char *)&control->pat_parameters[index];
  else
    return (unsigned char *)0x00000000;
}

void frame_buffer_control_update(TFBControl *control)
{
  unsigned char i;

  for(i=0;i<FBC_MAX_PATTERN_FUNC;i++)
  {
    if(control->pat_functions[i]!=0x00000000)
      control->pat_functions[i](&control->pat_area[i],&control->pat_parameters[i],control->pat_buffer_id[i],control->period_ms,control);
  }
}

// copy functions
unsigned char frame_buffer_control_add_copy_function(TFBControl *control,copy_func_t function,void *data)
{
  unsigned char i;

  for(i=0;i<FBC_MAX_COPY_FUNC;i++)
  {
    // find the first empty pattern
    if(control->copy_functions[i]==0x00000000)
    {
      control->copy_functions[i]=function;
      control->copy_data[i]=data;
      control->num_copy_functions++;
      return i;
    }
  }

  return 0xFF;
}

void frame_buffer_control_remove_copy_function(TFBControl *control,unsigned char index)
{
  if(control->num_copy_functions>0 && index<FBC_MAX_COPY_FUNC)
  {
    control->copy_functions[index]=0x00000000;
    control->copy_data[index]=0x0000000;
    control->num_copy_functions--;
  }
}

void frame_buffer_control_clear_copy_functions(TFBControl *control)
{
  unsigned char i;

  control->num_copy_functions=0;
  for(i=0;i<FBC_MAX_COPY_FUNC;i++)
  {
    control->copy_functions[i]=0x00000000;
    control->copy_data[i]=0x00000000;
  }
}

unsigned char frame_buffer_control_get_num_copy_functions(TFBControl *control)
{
  return control->num_copy_functions;
}
