#ifndef _SCHEDULER_H
#define _SCHEDULER_H

#ifdef __cplusplus
extern "C" {
#endif

#ifndef SCHEDULER_MAX_CHANNELS
  #define SCHEDULER_MAX_CHANNELS      4
#endif

#ifndef TIM_CHANNEL_1
  #define TIM_CHANNEL_1                      0x00000000U
#endif

#ifndef TIM_CHANNEL_2
  #define TIM_CHANNEL_2                      0x00000004U
#endif

#ifndef TIM_CHANNEL_3
  #define TIM_CHANNEL_3                      0x00000008U
#endif

#ifndef TIM_CHANNEL_4
  #define TIM_CHANNEL_4                      0x0000000CU
#endif

typedef enum {SCHED_CH1=0,SCHED_CH2=1,SCHED_CH3=2,SCHED_CH4=3} sched_channel_t;

typedef struct{
  void (*function)(void *data);
  unsigned char period_ms;
  unsigned int pulse;
  unsigned char enabled;
  unsigned char one_shot;
  void *data;
}TSchedulerChannel;

typedef struct{
  TSchedulerChannel channels[SCHEDULER_MAX_CHANNELS];
  unsigned char num_channels;
  unsigned short int prescaler;
  void (* start)(unsigned short int);
  void (* stop)(unsigned short int);
  void (* set_pulse)(unsigned short int,unsigned short int,unsigned char);
}TScheduler;

void scheduler_init(TScheduler *scheduler,unsigned char num_channels, unsigned short int prescaler);
void scheduler_interrupt(TScheduler *scheduler,sched_channel_t channel_id);
unsigned short int scheduler_get_pulse(TScheduler *scheduler, sched_channel_t channel_id);
void scheduler_set_channel(TScheduler *scheduler,sched_channel_t channel_id, void (*function)(void *data), unsigned char period_ms,void *data);
void scheduler_set_channel_one_shot(TScheduler *scheduler,sched_channel_t channel_id, void (*function)(void *data), unsigned char delay_ms,void *data);
void scheduler_enable_channel(TScheduler *scheduler,sched_channel_t channel_id);
void scheduler_change_period(TScheduler *scheduler,sched_channel_t channel_id,unsigned char period_ms);
void scheduler_change_delay(TScheduler *scheduler,sched_channel_t channel_id,unsigned char delay_ms);
void scheduler_disable_channel(TScheduler *scheduler,sched_channel_t channel_id);

#ifdef __cplusplus
}
#endif

#endif

