#include "scheduler.h"

extern unsigned int SystemCoreClock;

/* private functions */
unsigned short int scheduler_ms_to_pulse(TScheduler *scheduler,unsigned char period_ms)
{
  return (SystemCoreClock/(scheduler->prescaler*1000))*period_ms;
}

unsigned int scheduler_get_channel(sched_channel_t channel_id)
{
  switch(channel_id)
  {
    case SCHED_CH1: return TIM_CHANNEL_1;
    case SCHED_CH2: return TIM_CHANNEL_2;
    case SCHED_CH3: return TIM_CHANNEL_3;
    case SCHED_CH4: return TIM_CHANNEL_4;
    default: return TIM_CHANNEL_1;
  }
}

void scheduler_init(TScheduler *scheduler,unsigned char num_channels, unsigned short int prescaler)
{
  unsigned char i=0;

  for(i=0;i<SCHEDULER_MAX_CHANNELS;i++)
  {
    scheduler->channels[i].function=0x00000000;
    scheduler->channels[i].period_ms=0;
    scheduler->channels[i].enabled=0x00;
    scheduler->channels[i].one_shot=0x00;
  }
  scheduler->num_channels=num_channels;
  scheduler->prescaler=prescaler;
  scheduler->stop=0x00000000;
  scheduler->start=0x00000000;
  scheduler->set_pulse=0x00000000;
}

void scheduler_interrupt(TScheduler *scheduler,sched_channel_t channel_id)
{
  if(channel_id<scheduler->num_channels)
  {
    if(scheduler->channels[channel_id].one_shot==0x00)
    {
      if(scheduler->set_pulse!=0x00000000)
        scheduler->set_pulse(scheduler_get_channel(channel_id),scheduler->channels[channel_id].pulse,0x01);
      if(scheduler->channels[channel_id].function!=0x00000000)
        scheduler->channels[channel_id].function(scheduler->channels[channel_id].data);
    }
    else
    {
      if(scheduler->channels[channel_id].function!=0x00000000)
        scheduler->channels[channel_id].function(scheduler->channels[channel_id].data);
      scheduler_disable_channel(scheduler,channel_id);
    }
  }
}

void scheduler_set_channel(TScheduler *scheduler,sched_channel_t channel_id, void (*function)(void *data), unsigned char period_ms,void *data)
{
  if(channel_id<scheduler->num_channels)
  {
    if(scheduler->channels[channel_id].enabled==0x01)
    {
      if(scheduler->stop!=0x00000000)
        scheduler->stop(channel_id);
      scheduler->channels[channel_id].enabled=0x00;
    }
    scheduler->channels[channel_id].function=function;
    scheduler->channels[channel_id].period_ms=period_ms;
    scheduler->channels[channel_id].pulse=scheduler_ms_to_pulse(scheduler,period_ms);
    scheduler->channels[channel_id].data=data;
    scheduler->channels[channel_id].one_shot=0x00;
  }
}

void scheduler_set_channel_one_shot(TScheduler *scheduler,sched_channel_t channel_id, void (*function)(void *data), unsigned char delay_ms,void *data)
{
  if(channel_id<scheduler->num_channels)
  {
    if(scheduler->channels[channel_id].enabled==0x01)
    {
      if(scheduler->stop!=0x00000000)
        scheduler->stop(channel_id);
      scheduler->channels[channel_id].enabled=0x00;
    }
    scheduler->channels[channel_id].function=function;
    scheduler->channels[channel_id].period_ms=delay_ms;
    scheduler->channels[channel_id].pulse=scheduler_ms_to_pulse(scheduler,delay_ms);
    scheduler->channels[channel_id].data=data;
    scheduler->channels[channel_id].one_shot=0x01;
  }
}

void scheduler_enable_channel(TScheduler *scheduler,sched_channel_t channel_id)
{
  if(channel_id<scheduler->num_channels)
  {
    if(scheduler->channels[channel_id].enabled==0x00)
    {
      if(scheduler->set_pulse!=0x00000000)
        scheduler->set_pulse(scheduler_get_channel(channel_id),scheduler->channels[channel_id].pulse,0x00);
      if(scheduler->start!=0x00000000)
        scheduler->start(scheduler_get_channel(channel_id));
      scheduler->channels[channel_id].enabled=0x01;
    }
  }
}

void scheduler_change_period(TScheduler *scheduler,sched_channel_t channel_id,unsigned char period_ms)
{
  if(channel_id<scheduler->num_channels)
  {
    if(scheduler->channels[channel_id].one_shot==0x00)
    {
      scheduler->channels[channel_id].period_ms=period_ms;
      scheduler->channels[channel_id].pulse=scheduler_ms_to_pulse(scheduler,period_ms);
    }
  }
}

void scheduler_change_delay(TScheduler *scheduler,sched_channel_t channel_id,unsigned char delay_ms)
{
  if(channel_id<scheduler->num_channels)
  {
    if(scheduler->channels[channel_id].one_shot==0x01)
    {
      scheduler->channels[channel_id].period_ms=delay_ms;
      scheduler->channels[channel_id].pulse=scheduler_ms_to_pulse(scheduler,delay_ms);
    }
  }
}

void scheduler_disable_channel(TScheduler *scheduler,sched_channel_t channel_id)
{
  if(channel_id<scheduler->num_channels)
  {
    if(scheduler->channels[channel_id].enabled==0x01)
    {
      if(scheduler->stop!=0x00000000)
        scheduler->stop(scheduler_get_channel(channel_id));
      scheduler->channels[channel_id].enabled=0x00;
    }
  }
}

